
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class UnbControlInfoInterchangeIdQual(Enum):
    """An enumeration representing different categories.

    :cvar EDIFACTIDQUALNA: "EDIFACTIDQUAL_NA"
    :vartype EDIFACTIDQUALNA: str
    :cvar EDIFACTIDQUAL1: "EDIFACTIDQUAL_1"
    :vartype EDIFACTIDQUAL1: str
    :cvar EDIFACTIDQUAL4: "EDIFACTIDQUAL_4"
    :vartype EDIFACTIDQUAL4: str
    :cvar EDIFACTIDQUAL5: "EDIFACTIDQUAL_5"
    :vartype EDIFACTIDQUAL5: str
    :cvar EDIFACTIDQUAL8: "EDIFACTIDQUAL_8"
    :vartype EDIFACTIDQUAL8: str
    :cvar EDIFACTIDQUAL9: "EDIFACTIDQUAL_9"
    :vartype EDIFACTIDQUAL9: str
    :cvar EDIFACTIDQUAL12: "EDIFACTIDQUAL_12"
    :vartype EDIFACTIDQUAL12: str
    :cvar EDIFACTIDQUAL14: "EDIFACTIDQUAL_14"
    :vartype EDIFACTIDQUAL14: str
    :cvar EDIFACTIDQUAL18: "EDIFACTIDQUAL_18"
    :vartype EDIFACTIDQUAL18: str
    :cvar EDIFACTIDQUAL22: "EDIFACTIDQUAL_22"
    :vartype EDIFACTIDQUAL22: str
    :cvar EDIFACTIDQUAL30: "EDIFACTIDQUAL_30"
    :vartype EDIFACTIDQUAL30: str
    :cvar EDIFACTIDQUAL31: "EDIFACTIDQUAL_31"
    :vartype EDIFACTIDQUAL31: str
    :cvar EDIFACTIDQUAL33: "EDIFACTIDQUAL_33"
    :vartype EDIFACTIDQUAL33: str
    :cvar EDIFACTIDQUAL34: "EDIFACTIDQUAL_34"
    :vartype EDIFACTIDQUAL34: str
    :cvar EDIFACTIDQUAL51: "EDIFACTIDQUAL_51"
    :vartype EDIFACTIDQUAL51: str
    :cvar EDIFACTIDQUAL52: "EDIFACTIDQUAL_52"
    :vartype EDIFACTIDQUAL52: str
    :cvar EDIFACTIDQUAL53: "EDIFACTIDQUAL_53"
    :vartype EDIFACTIDQUAL53: str
    :cvar EDIFACTIDQUAL54: "EDIFACTIDQUAL_54"
    :vartype EDIFACTIDQUAL54: str
    :cvar EDIFACTIDQUAL55: "EDIFACTIDQUAL_55"
    :vartype EDIFACTIDQUAL55: str
    :cvar EDIFACTIDQUAL57: "EDIFACTIDQUAL_57"
    :vartype EDIFACTIDQUAL57: str
    :cvar EDIFACTIDQUAL58: "EDIFACTIDQUAL_58"
    :vartype EDIFACTIDQUAL58: str
    :cvar EDIFACTIDQUAL59: "EDIFACTIDQUAL_59"
    :vartype EDIFACTIDQUAL59: str
    :cvar EDIFACTIDQUAL61: "EDIFACTIDQUAL_61"
    :vartype EDIFACTIDQUAL61: str
    :cvar EDIFACTIDQUAL63: "EDIFACTIDQUAL_63"
    :vartype EDIFACTIDQUAL63: str
    :cvar EDIFACTIDQUAL65: "EDIFACTIDQUAL_65"
    :vartype EDIFACTIDQUAL65: str
    :cvar EDIFACTIDQUAL80: "EDIFACTIDQUAL_80"
    :vartype EDIFACTIDQUAL80: str
    :cvar EDIFACTIDQUAL82: "EDIFACTIDQUAL_82"
    :vartype EDIFACTIDQUAL82: str
    :cvar EDIFACTIDQUAL84: "EDIFACTIDQUAL_84"
    :vartype EDIFACTIDQUAL84: str
    :cvar EDIFACTIDQUAL85: "EDIFACTIDQUAL_85"
    :vartype EDIFACTIDQUAL85: str
    :cvar EDIFACTIDQUAL86: "EDIFACTIDQUAL_86"
    :vartype EDIFACTIDQUAL86: str
    :cvar EDIFACTIDQUAL87: "EDIFACTIDQUAL_87"
    :vartype EDIFACTIDQUAL87: str
    :cvar EDIFACTIDQUAL89: "EDIFACTIDQUAL_89"
    :vartype EDIFACTIDQUAL89: str
    :cvar EDIFACTIDQUAL90: "EDIFACTIDQUAL_90"
    :vartype EDIFACTIDQUAL90: str
    :cvar EDIFACTIDQUAL91: "EDIFACTIDQUAL_91"
    :vartype EDIFACTIDQUAL91: str
    :cvar EDIFACTIDQUAL92: "EDIFACTIDQUAL_92"
    :vartype EDIFACTIDQUAL92: str
    :cvar EDIFACTIDQUAL103: "EDIFACTIDQUAL_103"
    :vartype EDIFACTIDQUAL103: str
    :cvar EDIFACTIDQUAL128: "EDIFACTIDQUAL_128"
    :vartype EDIFACTIDQUAL128: str
    :cvar EDIFACTIDQUAL129: "EDIFACTIDQUAL_129"
    :vartype EDIFACTIDQUAL129: str
    :cvar EDIFACTIDQUAL144: "EDIFACTIDQUAL_144"
    :vartype EDIFACTIDQUAL144: str
    :cvar EDIFACTIDQUAL145: "EDIFACTIDQUAL_145"
    :vartype EDIFACTIDQUAL145: str
    :cvar EDIFACTIDQUAL146: "EDIFACTIDQUAL_146"
    :vartype EDIFACTIDQUAL146: str
    :cvar EDIFACTIDQUAL147: "EDIFACTIDQUAL_147"
    :vartype EDIFACTIDQUAL147: str
    :cvar EDIFACTIDQUAL148: "EDIFACTIDQUAL_148"
    :vartype EDIFACTIDQUAL148: str
    :cvar EDIFACTIDQUALZ01: "EDIFACTIDQUAL_Z01"
    :vartype EDIFACTIDQUALZ01: str
    :cvar EDIFACTIDQUALZZZ: "EDIFACTIDQUAL_ZZZ"
    :vartype EDIFACTIDQUALZZZ: str
    :cvar EDIFACTIDQUALZZ: "EDIFACTIDQUAL_ZZ"
    :vartype EDIFACTIDQUALZZ: str
    """

    EDIFACTIDQUALNA = "EDIFACTIDQUAL_NA"
    EDIFACTIDQUAL1 = "EDIFACTIDQUAL_1"
    EDIFACTIDQUAL4 = "EDIFACTIDQUAL_4"
    EDIFACTIDQUAL5 = "EDIFACTIDQUAL_5"
    EDIFACTIDQUAL8 = "EDIFACTIDQUAL_8"
    EDIFACTIDQUAL9 = "EDIFACTIDQUAL_9"
    EDIFACTIDQUAL12 = "EDIFACTIDQUAL_12"
    EDIFACTIDQUAL14 = "EDIFACTIDQUAL_14"
    EDIFACTIDQUAL18 = "EDIFACTIDQUAL_18"
    EDIFACTIDQUAL22 = "EDIFACTIDQUAL_22"
    EDIFACTIDQUAL30 = "EDIFACTIDQUAL_30"
    EDIFACTIDQUAL31 = "EDIFACTIDQUAL_31"
    EDIFACTIDQUAL33 = "EDIFACTIDQUAL_33"
    EDIFACTIDQUAL34 = "EDIFACTIDQUAL_34"
    EDIFACTIDQUAL51 = "EDIFACTIDQUAL_51"
    EDIFACTIDQUAL52 = "EDIFACTIDQUAL_52"
    EDIFACTIDQUAL53 = "EDIFACTIDQUAL_53"
    EDIFACTIDQUAL54 = "EDIFACTIDQUAL_54"
    EDIFACTIDQUAL55 = "EDIFACTIDQUAL_55"
    EDIFACTIDQUAL57 = "EDIFACTIDQUAL_57"
    EDIFACTIDQUAL58 = "EDIFACTIDQUAL_58"
    EDIFACTIDQUAL59 = "EDIFACTIDQUAL_59"
    EDIFACTIDQUAL61 = "EDIFACTIDQUAL_61"
    EDIFACTIDQUAL63 = "EDIFACTIDQUAL_63"
    EDIFACTIDQUAL65 = "EDIFACTIDQUAL_65"
    EDIFACTIDQUAL80 = "EDIFACTIDQUAL_80"
    EDIFACTIDQUAL82 = "EDIFACTIDQUAL_82"
    EDIFACTIDQUAL84 = "EDIFACTIDQUAL_84"
    EDIFACTIDQUAL85 = "EDIFACTIDQUAL_85"
    EDIFACTIDQUAL86 = "EDIFACTIDQUAL_86"
    EDIFACTIDQUAL87 = "EDIFACTIDQUAL_87"
    EDIFACTIDQUAL89 = "EDIFACTIDQUAL_89"
    EDIFACTIDQUAL90 = "EDIFACTIDQUAL_90"
    EDIFACTIDQUAL91 = "EDIFACTIDQUAL_91"
    EDIFACTIDQUAL92 = "EDIFACTIDQUAL_92"
    EDIFACTIDQUAL103 = "EDIFACTIDQUAL_103"
    EDIFACTIDQUAL128 = "EDIFACTIDQUAL_128"
    EDIFACTIDQUAL129 = "EDIFACTIDQUAL_129"
    EDIFACTIDQUAL144 = "EDIFACTIDQUAL_144"
    EDIFACTIDQUAL145 = "EDIFACTIDQUAL_145"
    EDIFACTIDQUAL146 = "EDIFACTIDQUAL_146"
    EDIFACTIDQUAL147 = "EDIFACTIDQUAL_147"
    EDIFACTIDQUAL148 = "EDIFACTIDQUAL_148"
    EDIFACTIDQUALZ01 = "EDIFACTIDQUAL_Z01"
    EDIFACTIDQUALZZZ = "EDIFACTIDQUAL_ZZZ"
    EDIFACTIDQUALZZ = "EDIFACTIDQUAL_ZZ"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, UnbControlInfoInterchangeIdQual._member_map_.values()
            )
        )


class UnbControlInfoPriority(Enum):
    """An enumeration representing different categories.

    :cvar NA: "NA"
    :vartype NA: str
    :cvar A: "A"
    :vartype A: str
    """

    NA = "NA"
    A = "A"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, UnbControlInfoPriority._member_map_.values())
        )


class UnbControlInfoReferencePasswordQualifier(Enum):
    """An enumeration representing different categories.

    :cvar NA: "NA"
    :vartype NA: str
    :cvar AA: "AA"
    :vartype AA: str
    :cvar BB: "BB"
    :vartype BB: str
    """

    NA = "NA"
    AA = "AA"
    BB = "BB"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                UnbControlInfoReferencePasswordQualifier._member_map_.values(),
            )
        )


class UnbControlInfoSyntaxId(Enum):
    """An enumeration representing different categories.

    :cvar UNOA: "UNOA"
    :vartype UNOA: str
    :cvar UNOB: "UNOB"
    :vartype UNOB: str
    :cvar UNOC: "UNOC"
    :vartype UNOC: str
    :cvar UNOD: "UNOD"
    :vartype UNOD: str
    :cvar UNOE: "UNOE"
    :vartype UNOE: str
    :cvar UNOF: "UNOF"
    :vartype UNOF: str
    """

    UNOA = "UNOA"
    UNOB = "UNOB"
    UNOC = "UNOC"
    UNOD = "UNOD"
    UNOE = "UNOE"
    UNOF = "UNOF"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, UnbControlInfoSyntaxId._member_map_.values())
        )


class UnbControlInfoSyntaxVersion(Enum):
    """An enumeration representing different categories.

    :cvar EDIFACTSYNTAXVERSION1: "EDIFACTSYNTAXVERSION_1"
    :vartype EDIFACTSYNTAXVERSION1: str
    :cvar EDIFACTSYNTAXVERSION2: "EDIFACTSYNTAXVERSION_2"
    :vartype EDIFACTSYNTAXVERSION2: str
    :cvar EDIFACTSYNTAXVERSION3: "EDIFACTSYNTAXVERSION_3"
    :vartype EDIFACTSYNTAXVERSION3: str
    """

    EDIFACTSYNTAXVERSION1 = "EDIFACTSYNTAXVERSION_1"
    EDIFACTSYNTAXVERSION2 = "EDIFACTSYNTAXVERSION_2"
    EDIFACTSYNTAXVERSION3 = "EDIFACTSYNTAXVERSION_3"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, UnbControlInfoSyntaxVersion._member_map_.values())
        )


class UnbControlInfoTestIndicator(Enum):
    """An enumeration representing different categories.

    :cvar EDIFACTTESTNA: "EDIFACTTEST_NA"
    :vartype EDIFACTTESTNA: str
    :cvar EDIFACTTEST1: "EDIFACTTEST_1"
    :vartype EDIFACTTEST1: str
    """

    EDIFACTTESTNA = "EDIFACTTEST_NA"
    EDIFACTTEST1 = "EDIFACTTEST_1"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, UnbControlInfoTestIndicator._member_map_.values())
        )


@JsonMap(
    {
        "ack_request": "ackRequest",
        "app_reference": "appReference",
        "comm_agreement": "commAgreement",
        "interchange_address": "interchangeAddress",
        "interchange_id": "interchangeId",
        "interchange_id_qual": "interchangeIdQual",
        "interchange_sub_address": "interchangeSubAddress",
        "reference_password": "referencePassword",
        "reference_password_qualifier": "referencePasswordQualifier",
        "syntax_id": "syntaxId",
        "syntax_version": "syntaxVersion",
        "test_indicator": "testIndicator",
    }
)
class UnbControlInfo(BaseModel):
    """UnbControlInfo

    :param ack_request: ack_request, defaults to None
    :type ack_request: bool, optional
    :param app_reference: app_reference, defaults to None
    :type app_reference: str, optional
    :param comm_agreement: comm_agreement, defaults to None
    :type comm_agreement: str, optional
    :param interchange_address: interchange_address, defaults to None
    :type interchange_address: str, optional
    :param interchange_id: interchange_id, defaults to None
    :type interchange_id: str, optional
    :param interchange_id_qual: interchange_id_qual, defaults to None
    :type interchange_id_qual: UnbControlInfoInterchangeIdQual, optional
    :param interchange_sub_address: interchange_sub_address, defaults to None
    :type interchange_sub_address: str, optional
    :param priority: priority, defaults to None
    :type priority: UnbControlInfoPriority, optional
    :param reference_password: reference_password, defaults to None
    :type reference_password: str, optional
    :param reference_password_qualifier: reference_password_qualifier, defaults to None
    :type reference_password_qualifier: UnbControlInfoReferencePasswordQualifier, optional
    :param syntax_id: syntax_id, defaults to None
    :type syntax_id: UnbControlInfoSyntaxId, optional
    :param syntax_version: syntax_version, defaults to None
    :type syntax_version: UnbControlInfoSyntaxVersion, optional
    :param test_indicator: test_indicator, defaults to None
    :type test_indicator: UnbControlInfoTestIndicator, optional
    """

    def __init__(
        self,
        ack_request: bool = SENTINEL,
        app_reference: str = SENTINEL,
        comm_agreement: str = SENTINEL,
        interchange_address: str = SENTINEL,
        interchange_id: str = SENTINEL,
        interchange_id_qual: UnbControlInfoInterchangeIdQual = SENTINEL,
        interchange_sub_address: str = SENTINEL,
        priority: UnbControlInfoPriority = SENTINEL,
        reference_password: str = SENTINEL,
        reference_password_qualifier: UnbControlInfoReferencePasswordQualifier = SENTINEL,
        syntax_id: UnbControlInfoSyntaxId = SENTINEL,
        syntax_version: UnbControlInfoSyntaxVersion = SENTINEL,
        test_indicator: UnbControlInfoTestIndicator = SENTINEL,
        **kwargs
    ):
        """UnbControlInfo

        :param ack_request: ack_request, defaults to None
        :type ack_request: bool, optional
        :param app_reference: app_reference, defaults to None
        :type app_reference: str, optional
        :param comm_agreement: comm_agreement, defaults to None
        :type comm_agreement: str, optional
        :param interchange_address: interchange_address, defaults to None
        :type interchange_address: str, optional
        :param interchange_id: interchange_id, defaults to None
        :type interchange_id: str, optional
        :param interchange_id_qual: interchange_id_qual, defaults to None
        :type interchange_id_qual: UnbControlInfoInterchangeIdQual, optional
        :param interchange_sub_address: interchange_sub_address, defaults to None
        :type interchange_sub_address: str, optional
        :param priority: priority, defaults to None
        :type priority: UnbControlInfoPriority, optional
        :param reference_password: reference_password, defaults to None
        :type reference_password: str, optional
        :param reference_password_qualifier: reference_password_qualifier, defaults to None
        :type reference_password_qualifier: UnbControlInfoReferencePasswordQualifier, optional
        :param syntax_id: syntax_id, defaults to None
        :type syntax_id: UnbControlInfoSyntaxId, optional
        :param syntax_version: syntax_version, defaults to None
        :type syntax_version: UnbControlInfoSyntaxVersion, optional
        :param test_indicator: test_indicator, defaults to None
        :type test_indicator: UnbControlInfoTestIndicator, optional
        """
        if ack_request is not SENTINEL:
            self.ack_request = ack_request
        if app_reference is not SENTINEL:
            self.app_reference = app_reference
        if comm_agreement is not SENTINEL:
            self.comm_agreement = comm_agreement
        if interchange_address is not SENTINEL:
            self.interchange_address = interchange_address
        if interchange_id is not SENTINEL:
            self.interchange_id = interchange_id
        if interchange_id_qual is not SENTINEL:
            self.interchange_id_qual = self._enum_matching(
                interchange_id_qual,
                UnbControlInfoInterchangeIdQual.list(),
                "interchange_id_qual",
            )
        if interchange_sub_address is not SENTINEL:
            self.interchange_sub_address = interchange_sub_address
        if priority is not SENTINEL:
            self.priority = self._enum_matching(
                priority, UnbControlInfoPriority.list(), "priority"
            )
        if reference_password is not SENTINEL:
            self.reference_password = reference_password
        if reference_password_qualifier is not SENTINEL:
            self.reference_password_qualifier = self._enum_matching(
                reference_password_qualifier,
                UnbControlInfoReferencePasswordQualifier.list(),
                "reference_password_qualifier",
            )
        if syntax_id is not SENTINEL:
            self.syntax_id = self._enum_matching(
                syntax_id, UnbControlInfoSyntaxId.list(), "syntax_id"
            )
        if syntax_version is not SENTINEL:
            self.syntax_version = self._enum_matching(
                syntax_version, UnbControlInfoSyntaxVersion.list(), "syntax_version"
            )
        if test_indicator is not SENTINEL:
            self.test_indicator = self._enum_matching(
                test_indicator, UnbControlInfoTestIndicator.list(), "test_indicator"
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "account_id": "accountId",
        "atom_id": "atomId",
        "atom_name": "atomName",
        "end_time": "endTime",
        "error_document_count": "errorDocumentCount",
        "error_type": "errorType",
        "errored_step_label": "erroredStepLabel",
        "errored_step_type": "erroredStepType",
        "event_date": "eventDate",
        "event_id": "eventId",
        "event_level": "eventLevel",
        "event_type": "eventType",
        "execution_id": "executionId",
        "inbound_document_count": "inboundDocumentCount",
        "outbound_document_count": "outboundDocumentCount",
        "process_id": "processId",
        "process_name": "processName",
        "record_date": "recordDate",
        "start_time": "startTime",
        "top_level_process_id": "topLevelProcessId",
        "update_date": "updateDate",
    }
)
class Event(BaseModel):
    """Event

    :param account_id: The ID of the account in which this event occurred.
    :type account_id: str
    :param atom_id: The ID of the Runtime on which the event occurred.
    :type atom_id: str
    :param atom_name: The name of the Runtime on which the event occurred.
    :type atom_name: str
    :param classification: The classification \(PROD or TEST\) associated with the environment in which the process ran, or to which the Runtime is attached. The classification PROD is returned in the QUERY result if you do not enable environments in the account.
    :type classification: str
    :param end_time: The end date and time of the run \(for applicable events\).
    :type end_time: str
    :param environment: The name of the environment in which the process ran, or to which the Runtime is attached. If you do not enable environments in the account, the QUERY does not return this field in the QUERY result.
    :type environment: str
    :param error: he error message \(for applicable events\).
    :type error: str
    :param error_document_count: The number of documents in an error status for the run \(for applicable events\)., defaults to None
    :type error_document_count: int, optional
    :param error_type: DOCUMENT, if the error applied to one or more documents, or PROCESS, if the error occurred at the process level \(for applicable events\).
    :type error_type: str
    :param errored_step_label: The label of the step in which the error occurred \(for applicable events\).
    :type errored_step_label: str
    :param errored_step_type: The type of the step in which the error occurred — for example, Start, Connector \(for applicable events\).
    :type errored_step_type: str
    :param event_date: The date and time the event occurred.
    :type event_date: str
    :param event_id: The ID of the event.
    :type event_id: str
    :param event_level: The notification level of the event \(INFO, WARNING, ERROR\).
    :type event_level: str
    :param event_type: The type of the event \(atom.status, process.execution, process.missedSchedule, user.notification\).
    :type event_type: str
    :param execution_id: The run ID of the process \(for applicable events\).
    :type execution_id: str
    :param inbound_document_count: The number of inbound documents for the run \(for applicable events\)., defaults to None
    :type inbound_document_count: int, optional
    :param outbound_document_count: The number of outbound documents for the run \(for applicable events\)., defaults to None
    :type outbound_document_count: int, optional
    :param process_id: The ID of the sub-process ONLY when using the ‘Notify’ shape with the 'Generate Platform Event' option and when the process is executed in the 'General' and 'Bridge' modes.
    :type process_id: str
    :param process_name: The name of the process \(for applicable events\).
    :type process_name: str
    :param record_date: The date and time the event recorded.
    :type record_date: str
    :param start_time: The start date and time of the run \(for applicable events\).
    :type start_time: str
    :param status: User-specified message for an event of type user.notification. For other types of events, the event status — for example, COMPLETE, ERROR.
    :type status: str
    :param title: For an event of type user.notification, the title of the originating step.
    :type title: str
    :param top_level_process_id: The ID of the parent process.
    :type top_level_process_id: str
    :param update_date: update_date
    :type update_date: str
    """

    def __init__(
        self,
        account_id: str,
        atom_id: str,
        atom_name: str,
        event_date: str,
        event_id: str,
        event_level: str,
        event_type: str,
        record_date: str,
        classification: str = SENTINEL,
        end_time: str = SENTINEL,
        environment: str = SENTINEL,
        error: str = SENTINEL,
        error_type: str = SENTINEL,
        errored_step_label: str = SENTINEL,
        errored_step_type: str = SENTINEL,
        execution_id: str = SENTINEL,
        process_id: str = SENTINEL,
        process_name: str = SENTINEL,
        start_time: str = SENTINEL,
        status: str = SENTINEL,
        title: str = SENTINEL,
        top_level_process_id: str = SENTINEL,
        update_date: str = SENTINEL,
        error_document_count: int = SENTINEL,
        inbound_document_count: int = SENTINEL,
        outbound_document_count: int = SENTINEL,
        **kwargs
    ):
        """Event

        :param account_id: The ID of the account in which this event occurred.
        :type account_id: str
        :param atom_id: The ID of the Runtime on which the event occurred.
        :type atom_id: str
        :param atom_name: The name of the Runtime on which the event occurred.
        :type atom_name: str
        :param classification: The classification \(PROD or TEST\) associated with the environment in which the process ran, or to which the Runtime is attached. The classification PROD is returned in the QUERY result if you do not enable environments in the account.
        :type classification: str
        :param end_time: The end date and time of the run \(for applicable events\).
        :type end_time: str
        :param environment: The name of the environment in which the process ran, or to which the Runtime is attached. If you do not enable environments in the account, the QUERY does not return this field in the QUERY result.
        :type environment: str
        :param error: he error message \(for applicable events\).
        :type error: str
        :param error_document_count: The number of documents in an error status for the run \(for applicable events\)., defaults to None
        :type error_document_count: int, optional
        :param error_type: DOCUMENT, if the error applied to one or more documents, or PROCESS, if the error occurred at the process level \(for applicable events\).
        :type error_type: str
        :param errored_step_label: The label of the step in which the error occurred \(for applicable events\).
        :type errored_step_label: str
        :param errored_step_type: The type of the step in which the error occurred — for example, Start, Connector \(for applicable events\).
        :type errored_step_type: str
        :param event_date: The date and time the event occurred.
        :type event_date: str
        :param event_id: The ID of the event.
        :type event_id: str
        :param event_level: The notification level of the event \(INFO, WARNING, ERROR\).
        :type event_level: str
        :param event_type: The type of the event \(atom.status, process.execution, process.missedSchedule, user.notification\).
        :type event_type: str
        :param execution_id: The run ID of the process \(for applicable events\).
        :type execution_id: str
        :param inbound_document_count: The number of inbound documents for the run \(for applicable events\)., defaults to None
        :type inbound_document_count: int, optional
        :param outbound_document_count: The number of outbound documents for the run \(for applicable events\)., defaults to None
        :type outbound_document_count: int, optional
        :param process_id: The ID of the sub-process ONLY when using the ‘Notify’ shape with the 'Generate Platform Event' option and when the process is executed in the 'General' and 'Bridge' modes.
        :type process_id: str
        :param process_name: The name of the process \(for applicable events\).
        :type process_name: str
        :param record_date: The date and time the event recorded.
        :type record_date: str
        :param start_time: The start date and time of the run \(for applicable events\).
        :type start_time: str
        :param status: User-specified message for an event of type user.notification. For other types of events, the event status — for example, COMPLETE, ERROR.
        :type status: str
        :param title: For an event of type user.notification, the title of the originating step.
        :type title: str
        :param top_level_process_id: The ID of the parent process.
        :type top_level_process_id: str
        :param update_date: update_date
        :type update_date: str
        """
        self.account_id = account_id
        self.atom_id = atom_id
        self.atom_name = atom_name
        if classification is not SENTINEL:
            self.classification = classification
        if end_time is not SENTINEL:
            self.end_time = end_time
        if environment is not SENTINEL:
            self.environment = environment
        if error is not SENTINEL:
            self.error = error
        if error_document_count is not SENTINEL:
            self.error_document_count = error_document_count
        if error_type is not SENTINEL:
            self.error_type = error_type
        if errored_step_label is not SENTINEL:
            self.errored_step_label = errored_step_label
        if errored_step_type is not SENTINEL:
            self.errored_step_type = errored_step_type
        self.event_date = event_date
        self.event_id = event_id
        self.event_level = event_level
        self.event_type = event_type
        if execution_id is not SENTINEL:
            self.execution_id = execution_id
        if inbound_document_count is not SENTINEL:
            self.inbound_document_count = inbound_document_count
        if outbound_document_count is not SENTINEL:
            self.outbound_document_count = outbound_document_count
        if process_id is not SENTINEL:
            self.process_id = process_id
        if process_name is not SENTINEL:
            self.process_name = process_name
        self.record_date = record_date
        if start_time is not SENTINEL:
            self.start_time = start_time
        if status is not SENTINEL:
            self.status = status
        if title is not SENTINEL:
            self.title = title
        if top_level_process_id is not SENTINEL:
            self.top_level_process_id = top_level_process_id
        if update_date is not SENTINEL:
            self.update_date = update_date
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..event import EventService
from ...models import EventQueryResponse, EventQueryConfig


class EventServiceAsync(EventService):
    """
    Async Wrapper for EventServiceAsync
    """

    def query_event(
        self, request_body: EventQueryConfig = None
    ) -> Awaitable[Union[EventQueryResponse, str]]:
        return to_async(super().query_event)(request_body)

    def query_more_event(
        self, request_body: str
    ) -> Awaitable[Union[EventQueryResponse, str]]:
        return to_async(super().query_more_event)(request_body)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AccountSsoConfig,
    AccountSsoConfigBulkRequest,
    AccountSsoConfigBulkResponse,
)


class AccountSsoConfigService(BaseService):

    @cast_models
    def get_account_sso_config(self, id_: str) -> Union[AccountSsoConfig, str]:
        """Returns the Account Single Sign-on Configuration for the supplied account ID.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountSsoConfig, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountSSOConfig/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountSsoConfig._unmap(response)
        if content == "application/xml":
            return AccountSsoConfig._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_account_sso_config(
        self, id_: str, request_body: AccountSsoConfig = None
    ) -> Union[AccountSsoConfig, str]:
        """Updates the Account Single Sign-on Configuration for the supplied account ID.

        :param request_body: The request body., defaults to None
        :type request_body: AccountSsoConfig, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountSsoConfig, str]
        """

        Validator(AccountSsoConfig).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountSSOConfig/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountSsoConfig._unmap(response)
        if content == "application/xml":
            return AccountSsoConfig._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_account_sso_config(self, id_: str) -> None:
        """Deletes the Account Single Sign-on Configuration for the supplied account ID.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountSSOConfig/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_account_sso_config(
        self, request_body: AccountSsoConfigBulkRequest = None
    ) -> Union[AccountSsoConfigBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: AccountSsoConfigBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountSsoConfigBulkResponse, str]
        """

        Validator(AccountSsoConfigBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountSSOConfig/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountSsoConfigBulkResponse._unmap(response)
        if content == "application/xml":
            return AccountSsoConfigBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..runtime_properties import RuntimePropertiesService
from ...models import (
    RuntimeProperties,
    AsyncOperationTokenResult,
    RuntimePropertiesAsyncResponse,
)


class RuntimePropertiesServiceAsync(RuntimePropertiesService):
    """
    Async Wrapper for RuntimePropertiesServiceAsync
    """

    def update_runtime_properties(
        self, id_: str, request_body: RuntimeProperties = None
    ) -> Awaitable[Union[RuntimeProperties, str]]:
        return to_async(super().update_runtime_properties)(id_, request_body)

    def async_get_runtime_properties(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_runtime_properties)(id_)

    def async_token_runtime_properties(
        self, token: str
    ) -> Awaitable[Union[RuntimePropertiesAsyncResponse, str]]:
        return to_async(super().async_token_runtime_properties)(token)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .publisher_packaged_component import PublisherPackagedComponent


@JsonMap({"publisher_packaged_component": "PublisherPackagedComponent"})
class PublisherPackagedComponents(BaseModel):
    """PublisherPackagedComponents

    :param publisher_packaged_component: publisher_packaged_component, defaults to None
    :type publisher_packaged_component: List[PublisherPackagedComponent], optional
    """

    def __init__(
        self,
        publisher_packaged_component: List[PublisherPackagedComponent] = SENTINEL,
        **kwargs,
    ):
        """PublisherPackagedComponents

        :param publisher_packaged_component: publisher_packaged_component, defaults to None
        :type publisher_packaged_component: List[PublisherPackagedComponent], optional
        """
        if publisher_packaged_component is not SENTINEL:
            self.publisher_packaged_component = self._define_list(
                publisher_packaged_component, PublisherPackagedComponent
            )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .shared_communication import SharedCommunication


@JsonMap({"shared_communication": "sharedCommunication"})
class SharedCommunications(BaseModel):
    """SharedCommunications

    :param shared_communication: shared_communication, defaults to None
    :type shared_communication: List[SharedCommunication], optional
    """

    def __init__(
        self, shared_communication: List[SharedCommunication] = SENTINEL, **kwargs
    ):
        """SharedCommunications

        :param shared_communication: shared_communication, defaults to None
        :type shared_communication: List[SharedCommunication], optional
        """
        if shared_communication is not SENTINEL:
            self.shared_communication = self._define_list(
                shared_communication, SharedCommunication
            )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .process_id import ProcessId


@JsonMap(
    {
        "process_id": "ProcessId",
        "id_": "id",
        "integration_pack_id": "integrationPackId",
        "integration_pack_override_name": "integrationPackOverrideName",
    }
)
class IntegrationPackInstance(BaseModel):
    """IntegrationPackInstance

    :param process_id: A list of process IDs associated with the integration pack instance, defaults to None
    :type process_id: List[ProcessId], optional
    :param id_: A unique ID assigned by the system to the installed instance of the integration pack. This field populates only if you install the integration pack in the requesting account., defaults to None
    :type id_: str, optional
    :param integration_pack_id: A unique ID assigned by the system to the integration pack., defaults to None
    :type integration_pack_id: str, optional
    :param integration_pack_override_name: The name of the installed instance of the integration pack. You can set this value only in the case of multi-install integration packs; its purpose is to distinguish between instances., defaults to None
    :type integration_pack_override_name: str, optional
    """

    def __init__(
        self,
        process_id: List[ProcessId] = SENTINEL,
        id_: str = SENTINEL,
        integration_pack_id: str = SENTINEL,
        integration_pack_override_name: str = SENTINEL,
        **kwargs,
    ):
        """IntegrationPackInstance

        :param process_id: A list of process IDs associated with the integration pack instance, defaults to None
        :type process_id: List[ProcessId], optional
        :param id_: A unique ID assigned by the system to the installed instance of the integration pack. This field populates only if you install the integration pack in the requesting account., defaults to None
        :type id_: str, optional
        :param integration_pack_id: A unique ID assigned by the system to the integration pack., defaults to None
        :type integration_pack_id: str, optional
        :param integration_pack_override_name: The name of the installed instance of the integration pack. You can set this value only in the case of multi-install integration packs; its purpose is to distinguish between instances., defaults to None
        :type integration_pack_override_name: str, optional
        """
        if process_id is not SENTINEL:
            self.process_id = self._define_list(process_id, ProcessId)
        if id_ is not SENTINEL:
            self.id_ = id_
        if integration_pack_id is not SENTINEL:
            self.integration_pack_id = integration_pack_id
        if integration_pack_override_name is not SENTINEL:
            self.integration_pack_override_name = integration_pack_override_name
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .generic_connector_record_expression import (
    GenericConnectorRecordExpression,
    GenericConnectorRecordExpressionGuard,
)
from .generic_connector_record_simple_expression import (
    GenericConnectorRecordSimpleExpression,
)
from .generic_connector_record_grouping_expression import (
    GenericConnectorRecordGroupingExpression,
)


@JsonMap({})
class GenericConnectorRecordQueryConfigQueryFilter(BaseModel):
    """GenericConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: GenericConnectorRecordExpression
    """

    def __init__(self, expression: GenericConnectorRecordExpression, **kwargs):
        """GenericConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: GenericConnectorRecordExpression
        """
        self.expression = GenericConnectorRecordExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class GenericConnectorRecordQueryConfig(BaseModel):
    """GenericConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: GenericConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: GenericConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """GenericConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: GenericConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, GenericConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SimpleLookupTableRow(BaseModel):
    """SimpleLookupTableRow

    :param ref1: ref1
    :type ref1: str
    :param ref2: ref2
    :type ref2: str
    """

    def __init__(self, ref1: str, ref2: str, **kwargs):
        """SimpleLookupTableRow

        :param ref1: ref1
        :type ref1: str
        :param ref2: ref2
        :type ref2: str
        """
        self.ref1 = ref1
        self.ref2 = ref2
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "message_id": "messageId",
        "message_id_qual": "messageIdQual",
    }
)
class CustomControlInfo(BaseModel):
    """CustomControlInfo

    :param message_id: message_id, defaults to None
    :type message_id: str, optional
    :param message_id_qual: message_id_qual, defaults to None
    :type message_id_qual: str, optional
    """

    def __init__(
        self,
        message_id: str = SENTINEL,
        message_id_qual: str = SENTINEL,
        **kwargs,
    ):
        """CustomControlInfo

        :param message_id: message_id, defaults to None
        :type message_id: str, optional
        :param message_id_qual: message_id_qual, defaults to None
        :type message_id_qual: str, optional
        """
        if message_id is not SENTINEL:
            self.message_id = message_id
        if message_id_qual is not SENTINEL:
            self.message_id_qual = message_id_qual
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .merge_request_detail import MergeRequestDetail


@JsonMap({"merge_request_detail": "MergeRequestDetail"})
class MergeRequestDetails(BaseModel):
    """MergeRequestDetails

    :param merge_request_detail: merge_request_detail, defaults to None
    :type merge_request_detail: List[MergeRequestDetail], optional
    """

    def __init__(
        self, merge_request_detail: List[MergeRequestDetail] = SENTINEL, **kwargs
    ):
        """MergeRequestDetails

        :param merge_request_detail: merge_request_detail, defaults to None
        :type merge_request_detail: List[MergeRequestDetail], optional
        """
        if merge_request_detail is not SENTINEL:
            self.merge_request_detail = self._define_list(
                merge_request_detail, MergeRequestDetail
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap(
    {"cluster_problem": "clusterProblem", "host_name": "hostName", "node_id": "nodeId"}
)
class NodeDetails(BaseModel):
    """NodeDetails

    :param cluster_problem: Lists any issues reported for nodes.
    :type cluster_problem: str
    :param host_name: Displays the external host name or IP of the machine where the node currently lives.
    :type host_name: str
    :param node_id: Displays the unique identifier associated with a particular node in the Runtime cluster or Runtime cloud. A star icon indicates the cluster's head node.
    :type node_id: str
    :param status: Lists the nodes in the Runtime cluster or Runtime cloud and displays some basic information about each node. By default, the nodes are sorted by `status`. You can sort the list by `status`, `nodeId`, or `hostName`.
    :type status: str
    """

    def __init__(
        self, cluster_problem: str, host_name: str, node_id: str, status: str, **kwargs
    ):
        """NodeDetails

        :param cluster_problem: Lists any issues reported for nodes.
        :type cluster_problem: str
        :param host_name: Displays the external host name or IP of the machine where the node currently lives.
        :type host_name: str
        :param node_id: Displays the unique identifier associated with a particular node in the Runtime cluster or Runtime cloud. A star icon indicates the cluster's head node.
        :type node_id: str
        :param status: Lists the nodes in the Runtime cluster or Runtime cloud and displays some basic information about each node. By default, the nodes are sorted by `status`. You can sort the list by `status`, `nodeId`, or `hostName`.
        :type status: str
        """
        self.cluster_problem = cluster_problem
        self.host_name = host_name
        self.node_id = node_id
        self.status = status
        self._kwargs = kwargs




from typing import Generator, Optional, Tuple
from ...transport.request import Request
from ...transport.response import Response


class BaseHandler:
    """
    A class for sending the request through the chain of handlers.

    :ivar BaseHandler _next_handler: The next handler in the chain.
    """

    def __init__(self):
        """
        Initialize a new instance of BaseHandler.
        """
        self._next_handler = None

    def handle(
        self, request: Request
    ) -> Tuple[Optional[Response], Optional[Exception]]:
        """
        Process the given request and return a response or an error.
        This method must be implemented by all subclasses.

        :param Request request: The request to handle.
        :return: The response and any error that occurred.
        :rtype: Tuple[Optional[Response], Optional[Exception]]
        """
        raise NotImplementedError()

    def stream(
        self, request: Request
    ) -> Generator[Tuple[Optional[Response], Optional[Exception]], None, None]:
        """
        Stream the given request and return a response or an error.
        This method must be implemented by all subclasses.

        :param Request request: The request to stream.
        :return: The response and any error that occurred.
        :rtype: Generator[Tuple[Optional[Response], Optional[Exception]], None, None]
        """
        raise NotImplementedError()

    def set_next(self, handler: "BaseHandler"):
        """
        Set the next handler in the chain.

        :param BaseHandler handler: The next handler.
        """
        self._next_handler = handler




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extension_browse_data import MapExtensionBrowseData


@JsonMap(
    {
        "destination_field_set": "DestinationFieldSet",
        "source_field_set": "SourceFieldSet",
        "environment_id": "environmentId",
        "extension_group_id": "extensionGroupId",
        "id_": "id",
        "map_id": "mapId",
        "process_id": "processId",
    }
)
class EnvironmentMapExtensionsSummary(BaseModel):
    """EnvironmentMapExtensionsSummary

    :param destination_field_set: Fields defining the credentials for connecting to the external service for the purpose of reimporting the source of destination profile to retrieve custom fields. You use these fields in the Environment Map Extension object's EXECUTE action.
    :type destination_field_set: MapExtensionBrowseData
    :param source_field_set: Fields defining the credentials for connecting to the external service for the purpose of reimporting the source of destination profile to retrieve custom fields. You use these fields in the Environment Map Extension object's EXECUTE action.
    :type source_field_set: MapExtensionBrowseData
    :param environment_id: The ID of the environment., defaults to None
    :type environment_id: str, optional
    :param extension_group_id: If applicable, the ID of the multi-install integration pack to which the extensible map applies., defaults to None
    :type extension_group_id: str, optional
    :param id_: The ID of the object.This is a conceptual ID synthesized from the IDs of the:\<br /\>-   Map\<br /\>-   Process\<br /\>-   Multi-install integration pack \(extensionGroupId\), if applicable\<br /\>-   Environment\<br /\>After obtaining this value with a QUERY operation, you can retrieve or update the extensible map by specifying the ID in a GET or UPDATE operation on an Environment Map Extension object, defaults to None
    :type id_: str, optional
    :param map_id: The ID of the extensible map., defaults to None
    :type map_id: str, optional
    :param name: The name of the extensible map. This name includes the source object definition name and the destination object definition name, separated by a hyphen., defaults to None
    :type name: str, optional
    :param process_id: The ID of the process., defaults to None
    :type process_id: str, optional
    """

    def __init__(
        self,
        destination_field_set: MapExtensionBrowseData,
        source_field_set: MapExtensionBrowseData,
        environment_id: str = SENTINEL,
        extension_group_id: str = SENTINEL,
        id_: str = SENTINEL,
        map_id: str = SENTINEL,
        name: str = SENTINEL,
        process_id: str = SENTINEL,
        **kwargs,
    ):
        """EnvironmentMapExtensionsSummary

        :param destination_field_set: Fields defining the credentials for connecting to the external service for the purpose of reimporting the source of destination profile to retrieve custom fields. You use these fields in the Environment Map Extension object's EXECUTE action.
        :type destination_field_set: MapExtensionBrowseData
        :param source_field_set: Fields defining the credentials for connecting to the external service for the purpose of reimporting the source of destination profile to retrieve custom fields. You use these fields in the Environment Map Extension object's EXECUTE action.
        :type source_field_set: MapExtensionBrowseData
        :param environment_id: The ID of the environment., defaults to None
        :type environment_id: str, optional
        :param extension_group_id: If applicable, the ID of the multi-install integration pack to which the extensible map applies., defaults to None
        :type extension_group_id: str, optional
        :param id_: The ID of the object.This is a conceptual ID synthesized from the IDs of the:\<br /\>-   Map\<br /\>-   Process\<br /\>-   Multi-install integration pack \(extensionGroupId\), if applicable\<br /\>-   Environment\<br /\>After obtaining this value with a QUERY operation, you can retrieve or update the extensible map by specifying the ID in a GET or UPDATE operation on an Environment Map Extension object, defaults to None
        :type id_: str, optional
        :param map_id: The ID of the extensible map., defaults to None
        :type map_id: str, optional
        :param name: The name of the extensible map. This name includes the source object definition name and the destination object definition name, separated by a hyphen., defaults to None
        :type name: str, optional
        :param process_id: The ID of the process., defaults to None
        :type process_id: str, optional
        """
        self.destination_field_set = self._define_object(
            destination_field_set, MapExtensionBrowseData
        )
        self.source_field_set = self._define_object(
            source_field_set, MapExtensionBrowseData
        )
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if extension_group_id is not SENTINEL:
            self.extension_group_id = extension_group_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if map_id is not SENTINEL:
            self.map_id = map_id
        if name is not SENTINEL:
            self.name = name
        if process_id is not SENTINEL:
            self.process_id = process_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .integration_pack_atom_attachment_simple_expression import (
    IntegrationPackAtomAttachmentSimpleExpression,
)
from .integration_pack_atom_attachment_grouping_expression import (
    IntegrationPackAtomAttachmentGroupingExpression,
)


class IntegrationPackAtomAttachmentExpressionGuard(OneOfBaseModel):
    class_list = {
        "IntegrationPackAtomAttachmentSimpleExpression": IntegrationPackAtomAttachmentSimpleExpression,
        "IntegrationPackAtomAttachmentGroupingExpression": IntegrationPackAtomAttachmentGroupingExpression,
    }


IntegrationPackAtomAttachmentExpression = Union[
    IntegrationPackAtomAttachmentSimpleExpression,
    IntegrationPackAtomAttachmentGroupingExpression,
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Folder,
    FolderBulkRequest,
    FolderBulkResponse,
    FolderQueryConfig,
    FolderQueryResponse,
)


class FolderService(BaseService):

    @cast_models
    def create_folder(self, request_body: Folder = None) -> Union[Folder, str]:
        """- When using the CREATE operation, you can create a new folder within the parent folder.

        - When creating a new folder, a name is required but PermittedRoles are optional. Unless it includes a list of UserRoles, in which case the GUID is required for the UserRole.

        - `parentId` must be a valid, non-deleted folder. If omitted or blank, it defaults to the root folder.

        - To Restore a folder you need to use the CREATE operation call, using a valid GUID for a deleted item. This will also restore any deleted components within that folder.

        :param request_body: The request body., defaults to None
        :type request_body: Folder, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Folder, str]
        """

        Validator(Folder).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Folder._unmap(response)
        if content == "application/xml":
            return Folder._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_folder(self, id_: str) -> Union[Folder, str]:
        """Retrieves the folder with the particular ID.

        :param id_: Required. Read only. The Boomi-generated, unique identifier of the folder.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Folder, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Folder._unmap(response)
        if content == "application/xml":
            return Folder._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_folder(
        self, id_: str, request_body: Folder = None
    ) -> Union[Folder, str]:
        """You can update by changing the name of the folder and following the same considerations for the CREATE parameters.

        :param request_body: The request body., defaults to None
        :type request_body: Folder, optional
        :param id_: Required. Read only. The Boomi-generated, unique identifier of the folder.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Folder, str]
        """

        Validator(Folder).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Folder._unmap(response)
        if content == "application/xml":
            return Folder._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_folder(self, id_: str) -> None:
        """- Deleting a folder will delete the folder and its contents including all components and sub-folders.
         - The root folder cannot be deleted.
         - Folders containing actively deployed processes or other deployable components cannot be deleted.
         >**Note:** You can restore a deleted folder by requesting a CREATE operation and specifying the ID of the deleted folder.

        :param id_: Required. Read only. The Boomi-generated, unique identifier of the folder.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_folder(
        self, request_body: FolderBulkRequest = None
    ) -> Union[FolderBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: FolderBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[FolderBulkResponse, str]
        """

        Validator(FolderBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return FolderBulkResponse._unmap(response)
        if content == "application/xml":
            return FolderBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_folder(
        self, request_body: FolderQueryConfig = None
    ) -> Union[FolderQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        - You can perform the QUERY operation for the Folder object by id, name, fullPath and deleted.

        - The QUERY MORE operation is also available for the Folder object.

        - You can perform an empty QUERY to return all folders.

        - If no filter is specified, both non-deleted and deleted records are returned.

        :param request_body: The request body., defaults to None
        :type request_body: FolderQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[FolderQueryResponse, str]
        """

        Validator(FolderQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return FolderQueryResponse._unmap(response)
        if content == "application/xml":
            return FolderQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_folder(self, request_body: str) -> Union[FolderQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[FolderQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Folder/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return FolderQueryResponse._unmap(response)
        if content == "application/xml":
            return FolderQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "mime_passthrough": "mimePassthrough",
        "object_name": "objectName",
        "operation_type": "operationType",
        "use_default_listen_options": "useDefaultListenOptions",
    }
)
class HttpListenOptions(BaseModel):
    """HttpListenOptions

    :param mime_passthrough: mime_passthrough, defaults to None
    :type mime_passthrough: bool, optional
    :param object_name: object_name, defaults to None
    :type object_name: str, optional
    :param operation_type: operation_type, defaults to None
    :type operation_type: str, optional
    :param password: password, defaults to None
    :type password: str, optional
    :param use_default_listen_options: use_default_listen_options, defaults to None
    :type use_default_listen_options: bool, optional
    :param username: username, defaults to None
    :type username: str, optional
    """

    def __init__(
        self,
        mime_passthrough: bool = SENTINEL,
        object_name: str = SENTINEL,
        operation_type: str = SENTINEL,
        password: str = SENTINEL,
        use_default_listen_options: bool = SENTINEL,
        username: str = SENTINEL,
        **kwargs
    ):
        """HttpListenOptions

        :param mime_passthrough: mime_passthrough, defaults to None
        :type mime_passthrough: bool, optional
        :param object_name: object_name, defaults to None
        :type object_name: str, optional
        :param operation_type: operation_type, defaults to None
        :type operation_type: str, optional
        :param password: password, defaults to None
        :type password: str, optional
        :param use_default_listen_options: use_default_listen_options, defaults to None
        :type use_default_listen_options: bool, optional
        :param username: username, defaults to None
        :type username: str, optional
        """
        if mime_passthrough is not SENTINEL:
            self.mime_passthrough = mime_passthrough
        if object_name is not SENTINEL:
            self.object_name = object_name
        if operation_type is not SENTINEL:
            self.operation_type = operation_type
        if password is not SENTINEL:
            self.password = password
        if use_default_listen_options is not SENTINEL:
            self.use_default_listen_options = use_default_listen_options
        if username is not SENTINEL:
            self.username = username
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .integration_pack_instance_expression import (
    IntegrationPackInstanceExpression,
    IntegrationPackInstanceExpressionGuard,
)
from .integration_pack_instance_simple_expression import (
    IntegrationPackInstanceSimpleExpression,
)
from .integration_pack_instance_grouping_expression import (
    IntegrationPackInstanceGroupingExpression,
)


@JsonMap({})
class IntegrationPackInstanceQueryConfigQueryFilter(BaseModel):
    """IntegrationPackInstanceQueryConfigQueryFilter

    :param expression: expression
    :type expression: IntegrationPackInstanceExpression
    """

    def __init__(self, expression: IntegrationPackInstanceExpression, **kwargs):
        """IntegrationPackInstanceQueryConfigQueryFilter

        :param expression: expression
        :type expression: IntegrationPackInstanceExpression
        """
        self.expression = IntegrationPackInstanceExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class IntegrationPackInstanceQueryConfig(BaseModel):
    """IntegrationPackInstanceQueryConfig

    :param query_filter: query_filter
    :type query_filter: IntegrationPackInstanceQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: IntegrationPackInstanceQueryConfigQueryFilter, **kwargs
    ):
        """IntegrationPackInstanceQueryConfig

        :param query_filter: query_filter
        :type query_filter: IntegrationPackInstanceQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, IntegrationPackInstanceQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "access_token": "accessToken",
        "access_token_key": "accessTokenKey",
        "client_id": "clientId",
        "client_secret": "clientSecret",
        "use_refresh_token": "useRefreshToken",
    }
)
class HttpoAuthCredentials(BaseModel):
    """HttpoAuthCredentials

    :param access_token: access_token, defaults to None
    :type access_token: str, optional
    :param access_token_key: access_token_key, defaults to None
    :type access_token_key: str, optional
    :param client_id: client_id, defaults to None
    :type client_id: str, optional
    :param client_secret: client_secret, defaults to None
    :type client_secret: str, optional
    :param use_refresh_token: use_refresh_token, defaults to None
    :type use_refresh_token: bool, optional
    """

    def __init__(
        self,
        access_token: str = SENTINEL,
        access_token_key: str = SENTINEL,
        client_id: str = SENTINEL,
        client_secret: str = SENTINEL,
        use_refresh_token: bool = SENTINEL,
        **kwargs
    ):
        """HttpoAuthCredentials

        :param access_token: access_token, defaults to None
        :type access_token: str, optional
        :param access_token_key: access_token_key, defaults to None
        :type access_token_key: str, optional
        :param client_id: client_id, defaults to None
        :type client_id: str, optional
        :param client_secret: client_secret, defaults to None
        :type client_secret: str, optional
        :param use_refresh_token: use_refresh_token, defaults to None
        :type use_refresh_token: bool, optional
        """
        if access_token is not SENTINEL:
            self.access_token = access_token
        if access_token_key is not SENTINEL:
            self.access_token_key = access_token_key
        if client_id is not SENTINEL:
            self.client_id = client_id
        if client_secret is not SENTINEL:
            self.client_secret = client_secret
        if use_refresh_token is not SENTINEL:
            self.use_refresh_token = use_refresh_token
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..publisher_integration_pack import PublisherIntegrationPackService
from ...models import (
    PublisherIntegrationPack,
    PublisherIntegrationPackBulkResponse,
    PublisherIntegrationPackBulkRequest,
    PublisherIntegrationPackQueryResponse,
    PublisherIntegrationPackQueryConfig,
)


class PublisherIntegrationPackServiceAsync(PublisherIntegrationPackService):
    """
    Async Wrapper for PublisherIntegrationPackServiceAsync
    """

    def create_publisher_integration_pack(
        self, request_body: PublisherIntegrationPack = None
    ) -> Awaitable[Union[PublisherIntegrationPack, str]]:
        return to_async(super().create_publisher_integration_pack)(request_body)

    def get_publisher_integration_pack(
        self, id_: str
    ) -> Awaitable[Union[PublisherIntegrationPack, str]]:
        return to_async(super().get_publisher_integration_pack)(id_)

    def update_publisher_integration_pack(
        self, id_: str, request_body: PublisherIntegrationPack = None
    ) -> Awaitable[Union[PublisherIntegrationPack, str]]:
        return to_async(super().update_publisher_integration_pack)(id_, request_body)

    def delete_publisher_integration_pack(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_publisher_integration_pack)(id_)

    def bulk_publisher_integration_pack(
        self, request_body: PublisherIntegrationPackBulkRequest = None
    ) -> Awaitable[Union[PublisherIntegrationPackBulkResponse, str, any]]:
        return to_async(super().bulk_publisher_integration_pack)(request_body)

    def query_publisher_integration_pack(
        self, request_body: PublisherIntegrationPackQueryConfig = None
    ) -> Awaitable[Union[PublisherIntegrationPackQueryResponse, str]]:
        return to_async(super().query_publisher_integration_pack)(request_body)

    def query_more_publisher_integration_pack(
        self, request_body: str
    ) -> Awaitable[Union[PublisherIntegrationPackQueryResponse, str]]:
        return to_async(super().query_more_publisher_integration_pack)(request_body)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import JavaUpgrade


class JavaUpgradeService(BaseService):

    @cast_models
    def create_java_upgrade(
        self, request_body: JavaUpgrade = None
    ) -> Union[JavaUpgrade, str]:
        """Download and run the Java upgrader script for a specified Runtime, Runtime cluster, Runtime cloud, Authentication Broker, or API Gateway.  Upgrades your selected container to Boomis latest supported version of Java.

         - After providing the endpoint and a request body that includes the containerID, the CREATE operation immediately upgrades the given container to Boomi's latest supported version of Java. After performing a CREATE operation, you can determine a successful upgrade when the **Update to use Java 11.<minor_version>** link no longer appears on the following pages:
          -  For Runtimes, Runtime clusters, and Runtime clouds — the **Runtime Information** panel (**Manage** > **Runtime Management** of the user interface).
          -  For Brokers (applicable for versions newer than 1.8.0_281-b09)— the **Broker Information** panel (**Configure Server** > **Authentication** of the user interface).
          -  For API Gateways — the **Gateway Information** panel (**Configure Server** > **Gateways** of the user interface).
         - You must have the **Runtime Management** privilege to perform the CREATE operation. If you have the **Runtime Management Read Access** privilege, you cannot use this operation to upgrade your container.
         - The container must be online to use the Upgrade Java operation.
        - The container must be eligible for upgrade.

         >**Important:** Only the node that runs upgrades (typically the head node) restarts automatically to run the updated Java version. Therefore, you must restart all other cluster nodes to install the upgrade.

        :param request_body: The request body., defaults to None
        :type request_body: JavaUpgrade, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[JavaUpgrade, str]
        """

        Validator(JavaUpgrade).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/JavaUpgrade",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return JavaUpgrade._unmap(response)
        if content == "application/xml":
            return JavaUpgrade._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Optional
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .packaged_component_expression import (
    PackagedComponentExpression,
    PackagedComponentExpressionGuard,
)
from .packaged_component_simple_expression import PackagedComponentSimpleExpression
from .packaged_component_grouping_expression import PackagedComponentGroupingExpression


@JsonMap({})
class PackagedComponentQueryConfigQueryFilter(BaseModel):
    """PackagedComponentQueryConfigQueryFilter

    :param expression: expression
    :type expression: PackagedComponentExpression
    """

    def __init__(self, expression: PackagedComponentExpression, **kwargs):
        """PackagedComponentQueryConfigQueryFilter

        :param expression: expression
        :type expression: PackagedComponentExpression
        """
        self.expression = PackagedComponentExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class PackagedComponentQueryConfig(BaseModel):
    """PackagedComponentQueryConfig

    :param query_filter: query_filter (optional)
    :type query_filter: Optional[PackagedComponentQueryConfigQueryFilter]
    """

    def __init__(self, query_filter: Optional[PackagedComponentQueryConfigQueryFilter] = None, **kwargs):
        """PackagedComponentQueryConfig

        :param query_filter: query_filter (optional)
        :type query_filter: Optional[PackagedComponentQueryConfigQueryFilter]
        """
        self.query_filter = self._define_object(
            query_filter, PackagedComponentQueryConfigQueryFilter
        ) if query_filter is not None else None
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .references import References


@JsonMap({})
class ComponentReference(BaseModel):
    """ComponentReference

    :param references: references, defaults to None
    :type references: List[References], optional
    """

    def __init__(self, references: List[References] = SENTINEL, **kwargs):
        """ComponentReference

        :param references: references, defaults to None
        :type references: List[References], optional
        """
        if references is not SENTINEL:
            # Handle both single reference (dict) and multiple references (list)
            if isinstance(references, dict):
                # Single reference from XML attributes - wrap in list
                self.references = self._define_list([references], References)
            elif isinstance(references, list):
                # Multiple references - normal case
                self.references = self._define_list(references, References)
            else:
                # Fallback to original behavior
                self.references = self._define_list(references, References)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_cloud_attachment_properties import (
    AccountCloudAttachmentPropertiesService,
)
from ...models import (
    AccountCloudAttachmentProperties,
    AsyncOperationTokenResult,
    AccountCloudAttachmentPropertiesAsyncResponse,
)


class AccountCloudAttachmentPropertiesServiceAsync(
    AccountCloudAttachmentPropertiesService
):
    """
    Async Wrapper for AccountCloudAttachmentPropertiesServiceAsync
    """

    def update_account_cloud_attachment_properties(
        self, id_: str, request_body: AccountCloudAttachmentProperties,
    ) -> Awaitable[Union[AccountCloudAttachmentProperties, str]]:
        return to_async(super().update_account_cloud_attachment_properties)(
            id_, request_body
        )

    def async_get_account_cloud_attachment_properties(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_account_cloud_attachment_properties)(id_)

    def async_token_account_cloud_attachment_properties(
        self, token: str
    ) -> Awaitable[Union[AccountCloudAttachmentPropertiesAsyncResponse, str]]:
        return to_async(super().async_token_account_cloud_attachment_properties)(token)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .account_group_simple_expression import AccountGroupSimpleExpression
from .account_group_grouping_expression import AccountGroupGroupingExpression


class AccountGroupExpressionGuard(OneOfBaseModel):
    class_list = {
        "AccountGroupSimpleExpression": AccountGroupSimpleExpression,
        "AccountGroupGroupingExpression": AccountGroupGroupingExpression,
    }


AccountGroupExpression = Union[
    AccountGroupSimpleExpression, AccountGroupGroupingExpression
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .runtime_cloud_expression import RuntimeCloudExpression, RuntimeCloudExpressionGuard
from .runtime_cloud_simple_expression import RuntimeCloudSimpleExpression
from .runtime_cloud_grouping_expression import RuntimeCloudGroupingExpression


@JsonMap({})
class RuntimeCloudQueryConfigQueryFilter(BaseModel):
    """RuntimeCloudQueryConfigQueryFilter

    :param expression: expression
    :type expression: RuntimeCloudExpression
    """

    def __init__(self, expression: RuntimeCloudExpression, **kwargs):
        """RuntimeCloudQueryConfigQueryFilter

        :param expression: expression
        :type expression: RuntimeCloudExpression
        """
        self.expression = RuntimeCloudExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class RuntimeCloudQueryConfig(BaseModel):
    """RuntimeCloudQueryConfig

    :param query_filter: query_filter
    :type query_filter: RuntimeCloudQueryConfigQueryFilter
    """

    def __init__(self, query_filter: RuntimeCloudQueryConfigQueryFilter, **kwargs):
        """RuntimeCloudQueryConfig

        :param query_filter: query_filter
        :type query_filter: RuntimeCloudQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, RuntimeCloudQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_map_extensions_summary_expression import (
    EnvironmentMapExtensionsSummaryExpression,
    EnvironmentMapExtensionsSummaryExpressionGuard,
)
from .environment_map_extensions_summary_simple_expression import (
    EnvironmentMapExtensionsSummarySimpleExpression,
)
from .environment_map_extensions_summary_grouping_expression import (
    EnvironmentMapExtensionsSummaryGroupingExpression,
)


@JsonMap({})
class EnvironmentMapExtensionsSummaryQueryConfigQueryFilter(BaseModel):
    """EnvironmentMapExtensionsSummaryQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentMapExtensionsSummaryExpression
    """

    def __init__(self, expression: EnvironmentMapExtensionsSummaryExpression, **kwargs):
        """EnvironmentMapExtensionsSummaryQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentMapExtensionsSummaryExpression
        """
        self.expression = EnvironmentMapExtensionsSummaryExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentMapExtensionsSummaryQueryConfig(BaseModel):
    """EnvironmentMapExtensionsSummaryQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentMapExtensionsSummaryQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: EnvironmentMapExtensionsSummaryQueryConfigQueryFilter,
        **kwargs,
    ):
        """EnvironmentMapExtensionsSummaryQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentMapExtensionsSummaryQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EnvironmentMapExtensionsSummaryQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "branch_name": "branchName",
        "component_id": "componentId",
        "component_type": "componentType",
        "component_version": "componentVersion",
        "created_by": "createdBy",
        "created_date": "createdDate",
        "fully_publicly_consumable": "fullyPubliclyConsumable",
        "package_id": "packageId",
        "package_version": "packageVersion",
    }
)
class PackagedComponent(BaseModel):
    """PackagedComponent

    :param branch_name: The name of the branch on which you want to create a packaged component.
    :type branch_name: str
    :param component_id: The ID of the component.The **component ID** is available on the **Revision History** dialog, which you can access from the **Build** page in the user interface.
    :type component_id: str
    :param component_type: The type of component: \<br /\>-   certificate\<br /\>-   customlibrary\<br /\>-   flowservice\<br /\>-   process\<br /\>-   processroute\<br /\>-   tpgroup\<br /\>-   webservice
    :type component_type: str
    :param component_version: When you save the component configuration change on the **Build** tab, this is the assigned Revision History number. You can find the component version in the **Revision History** dialog, which you can access from the **Build** page in the service., defaults to None
    :type component_version: int, optional
    :param created_by: The user ID of the person who created the packaged component.
    :type created_by: str
    :param created_date: The creation date and time of the packaged component.
    :type created_date: str
    :param deleted: deleted, defaults to None
    :type deleted: bool, optional
    :param fully_publicly_consumable: fully_publicly_consumable, defaults to None
    :type fully_publicly_consumable: bool, optional
    :param notes: Notes that describe the packaged component.
    :type notes: str
    :param package_id: The ID of the packaged component.
    :type package_id: str
    :param package_version: The user-defined version of the packaged component. Generates a value automatically based on the component's revision number if you do not specify a packaged component version.
    :type package_version: str
    :param shareable: \(For processes and API Service components only\) Identifies whether you can share the packaged component in the **Process Library** or as part of an integration pack.   \>**Note:** You cannot share processes that contain **Process Route** components in the **Process Library** or as part of an integration pack., defaults to None
    :type shareable: bool, optional
    """

    def __init__(
        self,
        component_id: str,
        component_type: str,
        package_version: str,
        package_id: str = SENTINEL,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        branch_name: str = SENTINEL,
        notes: str = SENTINEL,
        component_version: int = SENTINEL,
        deleted: bool = SENTINEL,
        fully_publicly_consumable: bool = SENTINEL,
        shareable: bool = SENTINEL,
        **kwargs
    ):
        """PackagedComponent

        :param branch_name: The name of the branch on which you want to create a packaged component.
        :type branch_name: str
        :param component_id: The ID of the component.The **component ID** is available on the **Revision History** dialog, which you can access from the **Build** page in the user interface.
        :type component_id: str
        :param component_type: The type of component: \<br /\>-   certificate\<br /\>-   customlibrary\<br /\>-   flowservice\<br /\>-   process\<br /\>-   processroute\<br /\>-   tpgroup\<br /\>-   webservice
        :type component_type: str
        :param component_version: When you save the component configuration change on the **Build** tab, this is the assigned Revision History number. You can find the component version in the **Revision History** dialog, which you can access from the **Build** page in the service., defaults to None
        :type component_version: int, optional
        :param created_by: The user ID of the person who created the packaged component.
        :type created_by: str
        :param created_date: The creation date and time of the packaged component.
        :type created_date: str
        :param deleted: deleted, defaults to None
        :type deleted: bool, optional
        :param fully_publicly_consumable: fully_publicly_consumable, defaults to None
        :type fully_publicly_consumable: bool, optional
        :param notes: Notes that describe the packaged component.
        :type notes: str
        :param package_id: The ID of the packaged component.
        :type package_id: str
        :param package_version: The user-defined version of the packaged component. Generates a value automatically based on the component's revision number if you do not specify a packaged component version.
        :type package_version: str
        :param shareable: \(For processes and API Service components only\) Identifies whether you can share the packaged component in the **Process Library** or as part of an integration pack.   \>**Note:** You cannot share processes that contain **Process Route** components in the **Process Library** or as part of an integration pack., defaults to None
        :type shareable: bool, optional
        """
        if branch_name is not SENTINEL:
            self.branch_name = branch_name
        self.component_id = component_id
        self.component_type = component_type
        if component_version is not SENTINEL:
            self.component_version = component_version
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if deleted is not SENTINEL:
            self.deleted = deleted
        if fully_publicly_consumable is not SENTINEL:
            self.fully_publicly_consumable = fully_publicly_consumable
        if notes is not SENTINEL:
            self.notes = notes
        if package_id is not SENTINEL:
            self.package_id = package_id
        self.package_version = package_version
        if shareable is not SENTINEL:
            self.shareable = shareable
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"account_id": "accountId", "atom_id": "atomId", "date_": "date"})
class ExecutionCountAccount(BaseModel):
    """ExecutionCountAccount

    :param account_id: The ID of the account under which the runs occurred., defaults to None
    :type account_id: str, optional
    :param atom_id: The ID of the Runtime on which the runs occurred., defaults to None
    :type atom_id: str, optional
    :param date_: The date on which the runs occurred. The time zone is UTC±00:00., defaults to None
    :type date_: str, optional
    :param failures: The count of failed runs., defaults to None
    :type failures: int, optional
    :param successes: The count of successful runs., defaults to None
    :type successes: int, optional
    """

    def __init__(
        self,
        account_id: str = SENTINEL,
        atom_id: str = SENTINEL,
        date_: str = SENTINEL,
        failures: int = SENTINEL,
        successes: int = SENTINEL,
        **kwargs
    ):
        """ExecutionCountAccount

        :param account_id: The ID of the account under which the runs occurred., defaults to None
        :type account_id: str, optional
        :param atom_id: The ID of the Runtime on which the runs occurred., defaults to None
        :type atom_id: str, optional
        :param date_: The date on which the runs occurred. The time zone is UTC±00:00., defaults to None
        :type date_: str, optional
        :param failures: The count of failed runs., defaults to None
        :type failures: int, optional
        :param successes: The count of successful runs., defaults to None
        :type successes: int, optional
        """
        if account_id is not SENTINEL:
            self.account_id = account_id
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if date_ is not SENTINEL:
            self.date_ = date_
        if failures is not SENTINEL:
            self.failures = failures
        if successes is not SENTINEL:
            self.successes = successes
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .deployed_expired_certificate_simple_expression import (
    DeployedExpiredCertificateSimpleExpression,
)
from .deployed_expired_certificate_grouping_expression import (
    DeployedExpiredCertificateGroupingExpression,
)


class DeployedExpiredCertificateExpressionGuard(OneOfBaseModel):
    class_list = {
        "DeployedExpiredCertificateSimpleExpression": DeployedExpiredCertificateSimpleExpression,
        "DeployedExpiredCertificateGroupingExpression": DeployedExpiredCertificateGroupingExpression,
    }


DeployedExpiredCertificateExpression = Union[
    DeployedExpiredCertificateSimpleExpression,
    DeployedExpiredCertificateGroupingExpression,
]




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_group import AccountGroupService
from ...models import (
    AccountGroup,
    AccountGroupBulkResponse,
    AccountGroupBulkRequest,
    AccountGroupQueryResponse,
    AccountGroupQueryConfig,
)


class AccountGroupServiceAsync(AccountGroupService):
    """
    Async Wrapper for AccountGroupServiceAsync
    """

    def create_account_group(
        self, request_body: AccountGroup = None
    ) -> Awaitable[Union[AccountGroup, str]]:
        return to_async(super().create_account_group)(request_body)

    def get_account_group(self, id_: str) -> Awaitable[Union[AccountGroup, str]]:
        return to_async(super().get_account_group)(id_)

    def update_account_group(
        self, id_: str, request_body: AccountGroup = None
    ) -> Awaitable[Union[AccountGroup, str]]:
        return to_async(super().update_account_group)(id_, request_body)

    def bulk_account_group(
        self, request_body: AccountGroupBulkRequest = None
    ) -> Awaitable[Union[AccountGroupBulkResponse, str]]:
        return to_async(super().bulk_account_group)(request_body)

    def query_account_group(
        self, request_body: AccountGroupQueryConfig = None
    ) -> Awaitable[Union[AccountGroupQueryResponse, str]]:
        return to_async(super().query_account_group)(request_body)

    def query_more_account_group(
        self, request_body: str
    ) -> Awaitable[Union[AccountGroupQueryResponse, str]]:
        return to_async(super().query_more_account_group)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_cloud_attachment_summary import AccountCloudAttachmentSummaryService
from ...models import (
    AccountCloudAttachmentSummary,
    AccountCloudAttachmentSummaryBulkResponse,
    AccountCloudAttachmentSummaryBulkRequest,
    AccountCloudAttachmentSummaryQueryResponse,
    AccountCloudAttachmentSummaryQueryConfig,
)


class AccountCloudAttachmentSummaryServiceAsync(AccountCloudAttachmentSummaryService):
    """Async wrapper for AccountCloudAttachmentSummaryService.

    .. note::
        The Cloud Management feature must be enabled for your Boomi account.
        Without it, all AccountCloudAttachmentSummary endpoints return HTTP 400
        with the message "The Cloud Management feature is not enabled for your
        account."  This response does not indicate a malformed SDK request.
    """

    def get_account_cloud_attachment_summary(
        self, id_: str
    ) -> Awaitable[Union[AccountCloudAttachmentSummary, str]]:
        return to_async(super().get_account_cloud_attachment_summary)(id_)

    def bulk_account_cloud_attachment_summary(
        self, request_body: AccountCloudAttachmentSummaryBulkRequest = None
    ) -> Awaitable[Union[AccountCloudAttachmentSummaryBulkResponse, str]]:
        return to_async(super().bulk_account_cloud_attachment_summary)(request_body)

    def query_account_cloud_attachment_summary(
        self, request_body: AccountCloudAttachmentSummaryQueryConfig = None
    ) -> Awaitable[Union[AccountCloudAttachmentSummaryQueryResponse, str]]:
        return to_async(super().query_account_cloud_attachment_summary)(request_body)

    def query_more_account_cloud_attachment_summary(
        self, request_body: str
    ) -> Awaitable[Union[AccountCloudAttachmentSummaryQueryResponse, str]]:
        return to_async(super().query_more_account_cloud_attachment_summary)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .trading_partner_component_expression import (
    TradingPartnerComponentExpression,
    TradingPartnerComponentExpressionGuard,
)
from .trading_partner_component_simple_expression import (
    TradingPartnerComponentSimpleExpression,
)
from .trading_partner_component_grouping_expression import (
    TradingPartnerComponentGroupingExpression,
)


@JsonMap({})
class TradingPartnerComponentQueryConfigQueryFilter(BaseModel):
    """TradingPartnerComponentQueryConfigQueryFilter

    :param expression: expression
    :type expression: TradingPartnerComponentExpression
    """

    def __init__(self, expression: TradingPartnerComponentExpression, **kwargs):
        """TradingPartnerComponentQueryConfigQueryFilter

        :param expression: expression
        :type expression: TradingPartnerComponentExpression
        """
        self.expression = TradingPartnerComponentExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class TradingPartnerComponentQueryConfig(BaseModel):
    """TradingPartnerComponentQueryConfig

    :param query_filter: query_filter
    :type query_filter: TradingPartnerComponentQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: TradingPartnerComponentQueryConfigQueryFilter, **kwargs
    ):
        """TradingPartnerComponentQueryConfig

        :param query_filter: query_filter
        :type query_filter: TradingPartnerComponentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, TradingPartnerComponentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class OdetteUnbControlInfoInterchangeIdQual(Enum):
    """An enumeration representing different categories.

    :cvar ODETTEIDQUALNA: "ODETTEIDQUAL_NA"
    :vartype ODETTEIDQUALNA: str
    :cvar ODETTEIDQUAL1: "ODETTEIDQUAL_1"
    :vartype ODETTEIDQUAL1: str
    :cvar ODETTEIDQUAL4: "ODETTEIDQUAL_4"
    :vartype ODETTEIDQUAL4: str
    :cvar ODETTEIDQUAL5: "ODETTEIDQUAL_5"
    :vartype ODETTEIDQUAL5: str
    :cvar ODETTEIDQUAL8: "ODETTEIDQUAL_8"
    :vartype ODETTEIDQUAL8: str
    :cvar ODETTEIDQUAL9: "ODETTEIDQUAL_9"
    :vartype ODETTEIDQUAL9: str
    :cvar ODETTEIDQUAL12: "ODETTEIDQUAL_12"
    :vartype ODETTEIDQUAL12: str
    :cvar ODETTEIDQUAL14: "ODETTEIDQUAL_14"
    :vartype ODETTEIDQUAL14: str
    :cvar ODETTEIDQUAL18: "ODETTEIDQUAL_18"
    :vartype ODETTEIDQUAL18: str
    :cvar ODETTEIDQUAL22: "ODETTEIDQUAL_22"
    :vartype ODETTEIDQUAL22: str
    :cvar ODETTEIDQUAL30: "ODETTEIDQUAL_30"
    :vartype ODETTEIDQUAL30: str
    :cvar ODETTEIDQUAL31: "ODETTEIDQUAL_31"
    :vartype ODETTEIDQUAL31: str
    :cvar ODETTEIDQUAL33: "ODETTEIDQUAL_33"
    :vartype ODETTEIDQUAL33: str
    :cvar ODETTEIDQUAL34: "ODETTEIDQUAL_34"
    :vartype ODETTEIDQUAL34: str
    :cvar ODETTEIDQUAL51: "ODETTEIDQUAL_51"
    :vartype ODETTEIDQUAL51: str
    :cvar ODETTEIDQUAL52: "ODETTEIDQUAL_52"
    :vartype ODETTEIDQUAL52: str
    :cvar ODETTEIDQUAL53: "ODETTEIDQUAL_53"
    :vartype ODETTEIDQUAL53: str
    :cvar ODETTEIDQUAL54: "ODETTEIDQUAL_54"
    :vartype ODETTEIDQUAL54: str
    :cvar ODETTEIDQUAL55: "ODETTEIDQUAL_55"
    :vartype ODETTEIDQUAL55: str
    :cvar ODETTEIDQUAL57: "ODETTEIDQUAL_57"
    :vartype ODETTEIDQUAL57: str
    :cvar ODETTEIDQUAL58: "ODETTEIDQUAL_58"
    :vartype ODETTEIDQUAL58: str
    :cvar ODETTEIDQUAL59: "ODETTEIDQUAL_59"
    :vartype ODETTEIDQUAL59: str
    :cvar ODETTEIDQUAL61: "ODETTEIDQUAL_61"
    :vartype ODETTEIDQUAL61: str
    :cvar ODETTEIDQUAL63: "ODETTEIDQUAL_63"
    :vartype ODETTEIDQUAL63: str
    :cvar ODETTEIDQUAL65: "ODETTEIDQUAL_65"
    :vartype ODETTEIDQUAL65: str
    :cvar ODETTEIDQUAL80: "ODETTEIDQUAL_80"
    :vartype ODETTEIDQUAL80: str
    :cvar ODETTEIDQUAL82: "ODETTEIDQUAL_82"
    :vartype ODETTEIDQUAL82: str
    :cvar ODETTEIDQUAL84: "ODETTEIDQUAL_84"
    :vartype ODETTEIDQUAL84: str
    :cvar ODETTEIDQUAL85: "ODETTEIDQUAL_85"
    :vartype ODETTEIDQUAL85: str
    :cvar ODETTEIDQUAL86: "ODETTEIDQUAL_86"
    :vartype ODETTEIDQUAL86: str
    :cvar ODETTEIDQUAL87: "ODETTEIDQUAL_87"
    :vartype ODETTEIDQUAL87: str
    :cvar ODETTEIDQUAL89: "ODETTEIDQUAL_89"
    :vartype ODETTEIDQUAL89: str
    :cvar ODETTEIDQUAL90: "ODETTEIDQUAL_90"
    :vartype ODETTEIDQUAL90: str
    :cvar ODETTEIDQUAL91: "ODETTEIDQUAL_91"
    :vartype ODETTEIDQUAL91: str
    :cvar ODETTEIDQUAL92: "ODETTEIDQUAL_92"
    :vartype ODETTEIDQUAL92: str
    :cvar ODETTEIDQUAL103: "ODETTEIDQUAL_103"
    :vartype ODETTEIDQUAL103: str
    :cvar ODETTEIDQUAL128: "ODETTEIDQUAL_128"
    :vartype ODETTEIDQUAL128: str
    :cvar ODETTEIDQUAL129: "ODETTEIDQUAL_129"
    :vartype ODETTEIDQUAL129: str
    :cvar ODETTEIDQUAL144: "ODETTEIDQUAL_144"
    :vartype ODETTEIDQUAL144: str
    :cvar ODETTEIDQUAL145: "ODETTEIDQUAL_145"
    :vartype ODETTEIDQUAL145: str
    :cvar ODETTEIDQUAL146: "ODETTEIDQUAL_146"
    :vartype ODETTEIDQUAL146: str
    :cvar ODETTEIDQUAL147: "ODETTEIDQUAL_147"
    :vartype ODETTEIDQUAL147: str
    :cvar ODETTEIDQUAL148: "ODETTEIDQUAL_148"
    :vartype ODETTEIDQUAL148: str
    :cvar ODETTEIDQUALZ01: "ODETTEIDQUAL_Z01"
    :vartype ODETTEIDQUALZ01: str
    :cvar ODETTEIDQUALZZZ: "ODETTEIDQUAL_ZZZ"
    :vartype ODETTEIDQUALZZZ: str
    :cvar ODETTEIDQUALZZ: "ODETTEIDQUAL_ZZ"
    :vartype ODETTEIDQUALZZ: str
    """

    ODETTEIDQUALNA = "ODETTEIDQUAL_NA"
    ODETTEIDQUAL1 = "ODETTEIDQUAL_1"
    ODETTEIDQUAL4 = "ODETTEIDQUAL_4"
    ODETTEIDQUAL5 = "ODETTEIDQUAL_5"
    ODETTEIDQUAL8 = "ODETTEIDQUAL_8"
    ODETTEIDQUAL9 = "ODETTEIDQUAL_9"
    ODETTEIDQUAL12 = "ODETTEIDQUAL_12"
    ODETTEIDQUAL14 = "ODETTEIDQUAL_14"
    ODETTEIDQUAL18 = "ODETTEIDQUAL_18"
    ODETTEIDQUAL22 = "ODETTEIDQUAL_22"
    ODETTEIDQUAL30 = "ODETTEIDQUAL_30"
    ODETTEIDQUAL31 = "ODETTEIDQUAL_31"
    ODETTEIDQUAL33 = "ODETTEIDQUAL_33"
    ODETTEIDQUAL34 = "ODETTEIDQUAL_34"
    ODETTEIDQUAL51 = "ODETTEIDQUAL_51"
    ODETTEIDQUAL52 = "ODETTEIDQUAL_52"
    ODETTEIDQUAL53 = "ODETTEIDQUAL_53"
    ODETTEIDQUAL54 = "ODETTEIDQUAL_54"
    ODETTEIDQUAL55 = "ODETTEIDQUAL_55"
    ODETTEIDQUAL57 = "ODETTEIDQUAL_57"
    ODETTEIDQUAL58 = "ODETTEIDQUAL_58"
    ODETTEIDQUAL59 = "ODETTEIDQUAL_59"
    ODETTEIDQUAL61 = "ODETTEIDQUAL_61"
    ODETTEIDQUAL63 = "ODETTEIDQUAL_63"
    ODETTEIDQUAL65 = "ODETTEIDQUAL_65"
    ODETTEIDQUAL80 = "ODETTEIDQUAL_80"
    ODETTEIDQUAL82 = "ODETTEIDQUAL_82"
    ODETTEIDQUAL84 = "ODETTEIDQUAL_84"
    ODETTEIDQUAL85 = "ODETTEIDQUAL_85"
    ODETTEIDQUAL86 = "ODETTEIDQUAL_86"
    ODETTEIDQUAL87 = "ODETTEIDQUAL_87"
    ODETTEIDQUAL89 = "ODETTEIDQUAL_89"
    ODETTEIDQUAL90 = "ODETTEIDQUAL_90"
    ODETTEIDQUAL91 = "ODETTEIDQUAL_91"
    ODETTEIDQUAL92 = "ODETTEIDQUAL_92"
    ODETTEIDQUAL103 = "ODETTEIDQUAL_103"
    ODETTEIDQUAL128 = "ODETTEIDQUAL_128"
    ODETTEIDQUAL129 = "ODETTEIDQUAL_129"
    ODETTEIDQUAL144 = "ODETTEIDQUAL_144"
    ODETTEIDQUAL145 = "ODETTEIDQUAL_145"
    ODETTEIDQUAL146 = "ODETTEIDQUAL_146"
    ODETTEIDQUAL147 = "ODETTEIDQUAL_147"
    ODETTEIDQUAL148 = "ODETTEIDQUAL_148"
    ODETTEIDQUALZ01 = "ODETTEIDQUAL_Z01"
    ODETTEIDQUALZZZ = "ODETTEIDQUAL_ZZZ"
    ODETTEIDQUALZZ = "ODETTEIDQUAL_ZZ"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OdetteUnbControlInfoInterchangeIdQual._member_map_.values(),
            )
        )


class OdetteUnbControlInfoPriority(Enum):
    """An enumeration representing different categories.

    :cvar NA: "NA"
    :vartype NA: str
    :cvar A: "A"
    :vartype A: str
    """

    NA = "NA"
    A = "A"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, OdetteUnbControlInfoPriority._member_map_.values())
        )


class OdetteUnbControlInfoReferencePasswordQualifier(Enum):
    """An enumeration representing different categories.

    :cvar NA: "NA"
    :vartype NA: str
    :cvar AA: "AA"
    :vartype AA: str
    :cvar BB: "BB"
    :vartype BB: str
    """

    NA = "NA"
    AA = "AA"
    BB = "BB"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OdetteUnbControlInfoReferencePasswordQualifier._member_map_.values(),
            )
        )


class OdetteUnbControlInfoSyntaxId(Enum):
    """An enumeration representing different categories.

    :cvar UNOA: "UNOA"
    :vartype UNOA: str
    :cvar UNOB: "UNOB"
    :vartype UNOB: str
    :cvar UNOC: "UNOC"
    :vartype UNOC: str
    :cvar UNOD: "UNOD"
    :vartype UNOD: str
    :cvar UNOE: "UNOE"
    :vartype UNOE: str
    :cvar UNOF: "UNOF"
    :vartype UNOF: str
    """

    UNOA = "UNOA"
    UNOB = "UNOB"
    UNOC = "UNOC"
    UNOD = "UNOD"
    UNOE = "UNOE"
    UNOF = "UNOF"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, OdetteUnbControlInfoSyntaxId._member_map_.values())
        )


class OdetteUnbControlInfoSyntaxVersion(Enum):
    """An enumeration representing different categories.

    :cvar ODETTESYNTAXVERSION1: "ODETTESYNTAXVERSION_1"
    :vartype ODETTESYNTAXVERSION1: str
    :cvar ODETTESYNTAXVERSION2: "ODETTESYNTAXVERSION_2"
    :vartype ODETTESYNTAXVERSION2: str
    :cvar ODETTESYNTAXVERSION3: "ODETTESYNTAXVERSION_3"
    :vartype ODETTESYNTAXVERSION3: str
    """

    ODETTESYNTAXVERSION1 = "ODETTESYNTAXVERSION_1"
    ODETTESYNTAXVERSION2 = "ODETTESYNTAXVERSION_2"
    ODETTESYNTAXVERSION3 = "ODETTESYNTAXVERSION_3"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OdetteUnbControlInfoSyntaxVersion._member_map_.values(),
            )
        )


class OdetteUnbControlInfoTestIndicator(Enum):
    """An enumeration representing different categories.

    :cvar ODETTETESTNA: "ODETTETEST_NA"
    :vartype ODETTETESTNA: str
    :cvar ODETTETEST1: "ODETTETEST_1"
    :vartype ODETTETEST1: str
    """

    ODETTETESTNA = "ODETTETEST_NA"
    ODETTETEST1 = "ODETTETEST_1"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OdetteUnbControlInfoTestIndicator._member_map_.values(),
            )
        )


@JsonMap(
    {
        "ack_request": "ackRequest",
        "app_reference": "appReference",
        "comm_agreement": "commAgreement",
        "interchange_address": "interchangeAddress",
        "interchange_id": "interchangeId",
        "interchange_id_qual": "interchangeIdQual",
        "interchange_sub_address": "interchangeSubAddress",
        "reference_password": "referencePassword",
        "reference_password_qualifier": "referencePasswordQualifier",
        "syntax_id": "syntaxId",
        "syntax_version": "syntaxVersion",
        "test_indicator": "testIndicator",
    }
)
class OdetteUnbControlInfo(BaseModel):
    """OdetteUnbControlInfo

    :param ack_request: ack_request, defaults to None
    :type ack_request: bool, optional
    :param app_reference: app_reference, defaults to None
    :type app_reference: str, optional
    :param comm_agreement: comm_agreement, defaults to None
    :type comm_agreement: str, optional
    :param interchange_address: interchange_address, defaults to None
    :type interchange_address: str, optional
    :param interchange_id: interchange_id, defaults to None
    :type interchange_id: str, optional
    :param interchange_id_qual: interchange_id_qual, defaults to None
    :type interchange_id_qual: OdetteUnbControlInfoInterchangeIdQual, optional
    :param interchange_sub_address: interchange_sub_address, defaults to None
    :type interchange_sub_address: str, optional
    :param priority: priority, defaults to None
    :type priority: OdetteUnbControlInfoPriority, optional
    :param reference_password: reference_password, defaults to None
    :type reference_password: str, optional
    :param reference_password_qualifier: reference_password_qualifier, defaults to None
    :type reference_password_qualifier: OdetteUnbControlInfoReferencePasswordQualifier, optional
    :param syntax_id: syntax_id, defaults to None
    :type syntax_id: OdetteUnbControlInfoSyntaxId, optional
    :param syntax_version: syntax_version, defaults to None
    :type syntax_version: OdetteUnbControlInfoSyntaxVersion, optional
    :param test_indicator: test_indicator, defaults to None
    :type test_indicator: OdetteUnbControlInfoTestIndicator, optional
    """

    def __init__(
        self,
        ack_request: bool = SENTINEL,
        app_reference: str = SENTINEL,
        comm_agreement: str = SENTINEL,
        interchange_address: str = SENTINEL,
        interchange_id: str = SENTINEL,
        interchange_id_qual: OdetteUnbControlInfoInterchangeIdQual = SENTINEL,
        interchange_sub_address: str = SENTINEL,
        priority: OdetteUnbControlInfoPriority = SENTINEL,
        reference_password: str = SENTINEL,
        reference_password_qualifier: OdetteUnbControlInfoReferencePasswordQualifier = SENTINEL,
        syntax_id: OdetteUnbControlInfoSyntaxId = SENTINEL,
        syntax_version: OdetteUnbControlInfoSyntaxVersion = SENTINEL,
        test_indicator: OdetteUnbControlInfoTestIndicator = SENTINEL,
        **kwargs
    ):
        """OdetteUnbControlInfo

        :param ack_request: ack_request, defaults to None
        :type ack_request: bool, optional
        :param app_reference: app_reference, defaults to None
        :type app_reference: str, optional
        :param comm_agreement: comm_agreement, defaults to None
        :type comm_agreement: str, optional
        :param interchange_address: interchange_address, defaults to None
        :type interchange_address: str, optional
        :param interchange_id: interchange_id, defaults to None
        :type interchange_id: str, optional
        :param interchange_id_qual: interchange_id_qual, defaults to None
        :type interchange_id_qual: OdetteUnbControlInfoInterchangeIdQual, optional
        :param interchange_sub_address: interchange_sub_address, defaults to None
        :type interchange_sub_address: str, optional
        :param priority: priority, defaults to None
        :type priority: OdetteUnbControlInfoPriority, optional
        :param reference_password: reference_password, defaults to None
        :type reference_password: str, optional
        :param reference_password_qualifier: reference_password_qualifier, defaults to None
        :type reference_password_qualifier: OdetteUnbControlInfoReferencePasswordQualifier, optional
        :param syntax_id: syntax_id, defaults to None
        :type syntax_id: OdetteUnbControlInfoSyntaxId, optional
        :param syntax_version: syntax_version, defaults to None
        :type syntax_version: OdetteUnbControlInfoSyntaxVersion, optional
        :param test_indicator: test_indicator, defaults to None
        :type test_indicator: OdetteUnbControlInfoTestIndicator, optional
        """
        if ack_request is not SENTINEL:
            self.ack_request = ack_request
        if app_reference is not SENTINEL:
            self.app_reference = app_reference
        if comm_agreement is not SENTINEL:
            self.comm_agreement = comm_agreement
        if interchange_address is not SENTINEL:
            self.interchange_address = interchange_address
        if interchange_id is not SENTINEL:
            self.interchange_id = interchange_id
        if interchange_id_qual is not SENTINEL:
            self.interchange_id_qual = self._enum_matching(
                interchange_id_qual,
                OdetteUnbControlInfoInterchangeIdQual.list(),
                "interchange_id_qual",
            )
        if interchange_sub_address is not SENTINEL:
            self.interchange_sub_address = interchange_sub_address
        if priority is not SENTINEL:
            self.priority = self._enum_matching(
                priority, OdetteUnbControlInfoPriority.list(), "priority"
            )
        if reference_password is not SENTINEL:
            self.reference_password = reference_password
        if reference_password_qualifier is not SENTINEL:
            self.reference_password_qualifier = self._enum_matching(
                reference_password_qualifier,
                OdetteUnbControlInfoReferencePasswordQualifier.list(),
                "reference_password_qualifier",
            )
        if syntax_id is not SENTINEL:
            self.syntax_id = self._enum_matching(
                syntax_id, OdetteUnbControlInfoSyntaxId.list(), "syntax_id"
            )
        if syntax_version is not SENTINEL:
            self.syntax_version = self._enum_matching(
                syntax_version,
                OdetteUnbControlInfoSyntaxVersion.list(),
                "syntax_version",
            )
        if test_indicator is not SENTINEL:
            self.test_indicator = self._enum_matching(
                test_indicator,
                OdetteUnbControlInfoTestIndicator.list(),
                "test_indicator",
            )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .trading_partner import TradingPartner


@JsonMap({"trading_partner": "tradingPartner"})
class TradingPartners(BaseModel):
    """TradingPartners

    :param trading_partner: trading_partner, defaults to None
    :type trading_partner: List[TradingPartner], optional
    """

    def __init__(self, trading_partner: List[TradingPartner] = SENTINEL, **kwargs):
        """TradingPartners

        :param trading_partner: trading_partner, defaults to None
        :type trading_partner: List[TradingPartner], optional
        """
        if trading_partner is not SENTINEL:
            self.trading_partner = self._define_list(trading_partner, TradingPartner)
        self._kwargs = kwargs




import re
import operator
from typing import Union, Any, Type, Pattern, get_args
from ...models.utils.sentinel import was_value_set
from ...models.utils.one_of_base_model import OneOfBaseModel


class Validator:
    """
    A simple validator class for validating the type, pattern, and other constraints of a value.

    :ivar Type[Any] _type: The expected type for the value.
    :ivar bool _is_optional: Flag indicating whether the value is optional.
    :ivar bool _is_nullable: Flag indicating whether the value can be null.
    :ivar bool _is_array: Flag indicating whether the value is an array.
    :ivar Pattern[str] _pattern: The regular expression pattern for validating the value.
    :ivar int _min_length: The minimum length for validating the value.
    :ivar int _max_length: The maximum length for validating the value.
    :ivar int _min: The minimum value for validating the value.
    :ivar bool _min_exclusive: Flag indicating whether the minimum value is exclusive.
    :ivar int _max: The maximum value for validating the value.
    :ivar bool _max_exclusive: Flag indicating whether the maximum value is exclusive.
    """

    def __init__(self, _type: Type[Any] = None):
        """
        Initializes a Validator instance.

        :param Type[Any] _type: The expected type for the value. Defaults to None.
        """
        self._type: Type[Any] = _type
        self._is_optional: bool = False
        self._is_nullable: bool = False
        self._is_array: bool = False
        self._pattern: Pattern[str] = None
        self._min_length: int = None
        self._max_length: int = None
        self._min: int = None
        self._min_exclusive: bool = False
        self._max: int = None
        self._max_exclusive: bool = False

    def is_array(self) -> "Validator":
        """
        Marks the value as an array.

        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._is_array = True
        return self

    def is_optional(self) -> "Validator":
        """
        Marks the value as optional.

        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._is_optional = True
        return self

    def is_nullable(self) -> "Validator":
        """
        Marks the value as nullable.

        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._is_nullable = True
        return self

    def pattern(self, pattern: str) -> "Validator":
        """
        Specifies a regular expression pattern for validating the value.

        :param str pattern: The regular expression pattern.
        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._pattern = re.compile(pattern)
        return self

    def min(self, min: int, exclusive=False) -> "Validator":
        """
        Specifies a minimum value for validating the value.

        :param int min: The minimum value to be validated against.
        :param bool exclusive: (optional) If set to True, the minimum value is not inclusive.
        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._min = min
        self._min_exclusive = exclusive
        return self

    def max(self, max: int, exclusive=False) -> "Validator":
        """
        Specifies a maximum value for validating the value.

        :param int max: The maximum value.
        :param bool exclusive: (optional) If set to True, the maximum value is not inclusive.
        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._max = max
        self._max_exclusive = exclusive
        return self

    def min_length(self, min_length: int) -> "Validator":
        """
        Specifies a minimum length for validating the value.

        :param int min_length: The minimum length to be validated against.
        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._min_length = min_length
        return self

    def max_length(self, max_length: int) -> "Validator":
        """
        Specifies a maximum length for validating the value.

        :param int max_length: The maximum length.
        :return: The Validator instance for method chaining.
        :rtype: Validator
        """
        self._max_length = max_length
        return self

    def validate(self, value: Any) -> None:
        """
        Validates the provided value based on the specified criteria.

        :param Any value: The input that needs to be checked
        :raises ValueError: If the value does not meet the specified validation criteria.
        """
        if not self._type:
            raise TypeError("Invalid type: No type specified")

        if self._is_nullable and value is None:
            return

        if self._is_optional and (not was_value_set(value) or value is None):
            return

        self._validate_type(value)
        self._validate_rules(value)

    def _validate_type(self, value: Any) -> None:
        """
        Validates the type of the value.

        :param Any value: The input that needs to be checked
        :raises ValueError: If the value does not meet the expected type.
        """
        if self._is_one_of_type(self._type):
            self._validate_one_of_type(value)
        elif self._is_array:
            self._validate_array_type(value)
        elif not self._match_type(value):
            raise TypeError(f"Invalid type: Expected {self._type}, got {type(value)}")

    def _validate_one_of_type(self, value: Any) -> None:
        """
        Validates oneOf model type.

        :param Any value: The input that needs to be checked
        :raises ValueError: If the value does not match the oneOf rules.
        """
        class_list = {
            arg.__name__: arg
            for arg in get_args(self._type)
            if hasattr(arg, "__name__")
        }
        OneOfBaseModel.class_list = class_list
        OneOfBaseModel.return_one_of(value)

    def _validate_array_type(self, value: Any) -> None:
        """
        Validates the type of an array value.

        :param Any value: The input that needs to be checked
        :raises ValueError: If the array items do not match the expected type.
        """
        if any(self._match_type(v) is False for v in value):
            raise TypeError(f"Invalid type: Expected {self._type}, got {type(value)}")

    def _match_type(self, value: Any) -> bool:
        """
        Checks if the value matches the expected type.

        :param Any value: The input that needs to be checked
        :raises ValueError: If the value does not match the expected type.
        """
        is_numeric = self._type is float and isinstance(value, int)
        if isinstance(value, self._type) or is_numeric:
            return True
        return False

    def _validate_rules(self, value: Any) -> None:
        """
        Validate the rules specified for the value.

        :param Any value: The input that needs to be validated
        :raises ValueError: If the value does not meet the specified validation criteria.
        """
        min_operator = operator.lt if self._min_exclusive else operator.le
        max_operator = operator.gt if self._max_exclusive else operator.ge

        if self._min is not None and not min_operator(self._min, value):
            raise ValueError(
                f"Invalid value: {value} is {'less than or equal to' if self._min_exclusive else 'less than'} {self._min}"
            )
        if self._max is not None and not max_operator(self._max, value):
            raise ValueError(
                f"Invalid value: {value} is {'greater than or equal to' if self._max_exclusive else 'greater than'} {self._max}"
            )
        if self._min_length is not None and len(value) < self._min_length:
            raise ValueError(
                f"Invalid value: the length of {value} is less than {self._min_length}"
            )
        if self._max_length is not None and len(value) > self._max_length:
            raise ValueError(
                f"Invalid value: the length of {value} is greater than {self._max_length}"
            )
        if self._pattern and not self._pattern.match(str(value)):
            raise ValueError(
                f"Invalid value: {value} does not match pattern {self._pattern}"
            )

    def _is_one_of_type(self, cls_type):
        """
        Checks if the provided type is a Union type.

        :param Type[Any] cls_type: The type to be checked.
        :return: True if the type is a Union type, False otherwise.
        :rtype: bool
        """
        return hasattr(cls_type, "__origin__") and cls_type.__origin__ is Union




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"id_": "id", "use_default": "useDefault"})
class PgpCertificate(BaseModel):
    """PgpCertificate

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param use_default: use_default, defaults to None
    :type use_default: bool, optional
    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(
        self,
        id_: str = SENTINEL,
        use_default: bool = SENTINEL,
        value: str = SENTINEL,
        **kwargs
    ):
        """PgpCertificate

        :param id_: id_, defaults to None
        :type id_: str, optional
        :param use_default: use_default, defaults to None
        :type use_default: bool, optional
        :param value: value, defaults to None
        :type value: str, optional
        """
        if id_ is not SENTINEL:
            self.id_ = id_
        if use_default is not SENTINEL:
            self.use_default = use_default
        if value is not SENTINEL:
            self.value = value
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "id_": "id",
        "name": "name",
    }
)
class RoleReference(BaseModel):
    """RoleReference

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: str = SENTINEL, name: str = SENTINEL, **kwargs):
        """RoleReference

        :param id_: id_, defaults to None
        :type id_: str, optional
        :param name: name, defaults to None
        :type name: str, optional
        """
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .partner_archiving import PartnerArchiving
from .partner_communication import PartnerCommunication


@JsonMap(
    {
        "partner_archiving": "PartnerArchiving",
        "partner_communication": "PartnerCommunication",
        "communication_type": "communicationType",
        "component_id": "componentId",
        "component_name": "componentName",
        "folder_id": "folderId",
        "folder_name": "folderName",
        "branch_id": "branchId",
        "branch_name": "branchName",
    }
)
class SharedCommunicationChannelComponent(BaseModel):
    """SharedCommunicationChannelComponent

    :param partner_archiving: partner_archiving
    :type partner_archiving: PartnerArchiving
    :param partner_communication: partner_communication
    :type partner_communication: PartnerCommunication
    :param communication_type: communication_type, defaults to None
    :type communication_type: str, optional
    :param component_id: component_id, defaults to None
    :type component_id: str, optional
    :param component_name: component_name, defaults to None
    :type component_name: str, optional
    :param deleted: deleted, defaults to None
    :type deleted: bool, optional
    :param description: description, defaults to None
    :type description: str, optional
    :param folder_id: folder_id, defaults to None
    :type folder_id: int, optional
    :param folder_name: folder_name, defaults to None
    :type folder_name: str, optional
    :param branch_id: branch_id, defaults to None
    :type branch_id: str, optional
    :param branch_name: branch_name, defaults to None
    :type branch_name: str, optional
    """

    def __init__(
        self,
        partner_archiving: PartnerArchiving,
        partner_communication: PartnerCommunication,
        communication_type: str = SENTINEL,
        component_id: str = SENTINEL,
        component_name: str = SENTINEL,
        deleted: bool = SENTINEL,
        description: str = SENTINEL,
        folder_id: int = SENTINEL,
        folder_name: str = SENTINEL,
        branch_id: str = SENTINEL,
        branch_name: str = SENTINEL,
        **kwargs,
    ):
        """SharedCommunicationChannelComponent

        :param partner_archiving: partner_archiving
        :type partner_archiving: PartnerArchiving
        :param partner_communication: partner_communication
        :type partner_communication: PartnerCommunication
        :param communication_type: communication_type, defaults to None
        :type communication_type: str, optional
        :param component_id: component_id, defaults to None
        :type component_id: str, optional
        :param component_name: component_name, defaults to None
        :type component_name: str, optional
        :param deleted: deleted, defaults to None
        :type deleted: bool, optional
        :param description: description, defaults to None
        :type description: str, optional
        :param folder_id: folder_id, defaults to None
        :type folder_id: int, optional
        :param folder_name: folder_name, defaults to None
        :type folder_name: str, optional
        :param branch_id: branch_id, defaults to None
        :type branch_id: str, optional
        :param branch_name: branch_name, defaults to None
        :type branch_name: str, optional
        """
        self.partner_archiving = self._define_object(
            partner_archiving, PartnerArchiving
        )
        self.partner_communication = self._define_object(
            partner_communication, PartnerCommunication
        )
        if communication_type is not SENTINEL:
            self.communication_type = communication_type
        if component_id is not SENTINEL:
            self.component_id = component_id
        if component_name is not SENTINEL:
            self.component_name = component_name
        if deleted is not SENTINEL:
            self.deleted = deleted
        if description is not SENTINEL:
            self.description = description
        if folder_id is not SENTINEL:
            self.folder_id = folder_id
        if folder_name is not SENTINEL:
            self.folder_name = folder_name
        if branch_id is not SENTINEL:
            self.branch_id = branch_id
        if branch_name is not SENTINEL:
            self.branch_name = branch_name
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class LogLevel(Enum):
    """An enumeration representing different categories.

    :cvar SEVERE: "SEVERE"
    :vartype SEVERE: str
    :cvar WARNING: "WARNING"
    :vartype WARNING: str
    :cvar INFO: "INFO"
    :vartype INFO: str
    :cvar CONFIG: "CONFIG"
    :vartype CONFIG: str
    :cvar FINE: "FINE"
    :vartype FINE: str
    :cvar FINER: "FINER"
    :vartype FINER: str
    :cvar FINEST: "FINEST"
    :vartype FINEST: str
    :cvar ALL: "ALL"
    :vartype ALL: str
    """

    SEVERE = "SEVERE"
    WARNING = "WARNING"
    INFO = "INFO"
    CONFIG = "CONFIG"
    FINE = "FINE"
    FINER = "FINER"
    FINEST = "FINEST"
    ALL = "ALL"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, LogLevel._member_map_.values()))


@JsonMap({"execution_id": "executionId", "log_level": "logLevel"})
class ProcessLog(BaseModel):
    """ProcessLog

    :param execution_id: The ID of the process run., defaults to None
    :type execution_id: str, optional
    :param log_level: The process execution log level with ALL being the default.   If you do not specify the log level, you receive all types of logs. The log level is case sensitive; you must use all uppercase letters., defaults to None
    :type log_level: LogLevel, optional
    """

    def __init__(
        self, execution_id: str = SENTINEL, log_level: LogLevel = SENTINEL, **kwargs
    ):
        """ProcessLog

        :param execution_id: The ID of the process run., defaults to None
        :type execution_id: str, optional
        :param log_level: The process execution log level with ALL being the default.   If you do not specify the log level, you receive all types of logs. The log level is case sensitive; you must use all uppercase letters., defaults to None
        :type log_level: LogLevel, optional
        """
        if execution_id is not SENTINEL:
            self.execution_id = execution_id
        if log_level is not SENTINEL:
            self.log_level = self._enum_matching(
                log_level, LogLevel.list(), "log_level"
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "account_id": "accountId",
        "cloud_id": "cloudId",
        "id_": "id",
        "max_atom_attachment": "maxAtomAttachment",
    }
)
class AccountCloudAttachmentQuota(BaseModel):
    """AccountCloudAttachmentQuota

    :param account_id: The ID of the account authorizing the call., defaults to None
    :type account_id: str, optional
    :param cloud_id: The ID of the Runtime cloud that you want to get, add, edit, or delete a Cloud quota., defaults to None
    :type cloud_id: str, optional
    :param id_: A unique ID generated for a newly created or recently updated Cloud quota \(using the Account Cloud Attachment quota object\). You can use this ID to get a Cloud quota for an account's specific Cloud ID., defaults to None
    :type id_: str, optional
    :param max_atom_attachment: The number of Runtime attachments that you want to set on the Cloud quota., defaults to None
    :type max_atom_attachment: int, optional
    """

    def __init__(
        self,
        account_id: str = SENTINEL,
        cloud_id: str = SENTINEL,
        id_: str = SENTINEL,
        max_atom_attachment: int = SENTINEL,
        **kwargs
    ):
        """AccountCloudAttachmentQuota

        :param account_id: The ID of the account authorizing the call., defaults to None
        :type account_id: str, optional
        :param cloud_id: The ID of the Runtime cloud that you want to get, add, edit, or delete a Cloud quota., defaults to None
        :type cloud_id: str, optional
        :param id_: A unique ID generated for a newly created or recently updated Cloud quota \(using the Account Cloud Attachment quota object\). You can use this ID to get a Cloud quota for an account's specific Cloud ID., defaults to None
        :type id_: str, optional
        :param max_atom_attachment: The number of Runtime attachments that you want to set on the Cloud quota., defaults to None
        :type max_atom_attachment: int, optional
        """
        if account_id is not SENTINEL:
            self.account_id = account_id
        if cloud_id is not SENTINEL:
            self.cloud_id = cloud_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if max_atom_attachment is not SENTINEL:
            self.max_atom_attachment = max_atom_attachment
        self._kwargs = kwargs




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class GenericConnectorRecordSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    """

    EQUALS = "EQUALS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                GenericConnectorRecordSimpleExpressionOperator._member_map_.values(),
            )
        )


class GenericConnectorRecordSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar ID: "id"
    :vartype ID: str
    :cvar EXECUTIONCONNECTORID: "executionConnectorId"
    :vartype EXECUTIONCONNECTORID: str
    :cvar EXECUTIONID: "executionId"
    :vartype EXECUTIONID: str
    :cvar CONNECTIONID: "connectionId"
    :vartype CONNECTIONID: str
    :cvar OPERATIONID: "operationId"
    :vartype OPERATIONID: str
    :cvar ACTIONTYPE: "actionType"
    :vartype ACTIONTYPE: str
    :cvar CONNECTORTYPE: "connectorType"
    :vartype CONNECTORTYPE: str
    :cvar ATOMID: "atomId"
    :vartype ATOMID: str
    :cvar DATEPROCESSED: "dateProcessed"
    :vartype DATEPROCESSED: str
    :cvar CONNECTIONNAME: "connectionName"
    :vartype CONNECTIONNAME: str
    :cvar OPERATIONNAME: "operationName"
    :vartype OPERATIONNAME: str
    :cvar ERRORMESSAGE: "errorMessage"
    :vartype ERRORMESSAGE: str
    :cvar STATUS: "status"
    :vartype STATUS: str
    :cvar DOCUMENTINDEX: "documentIndex"
    :vartype DOCUMENTINDEX: str
    :cvar INCREMENTALDOCUMENTINDEX: "incrementalDocumentIndex"
    :vartype INCREMENTALDOCUMENTINDEX: str
    :cvar SIZE: "size"
    :vartype SIZE: str
    :cvar STARTSHAPE: "startShape"
    :vartype STARTSHAPE: str
    :cvar RETRYABLE: "retryable"
    :vartype RETRYABLE: str
    """

    ID = "id"
    EXECUTIONCONNECTORID = "executionConnectorId"
    EXECUTIONID = "executionId"
    CONNECTIONID = "connectionId"
    OPERATIONID = "operationId"
    ACTIONTYPE = "actionType"
    CONNECTORTYPE = "connectorType"
    ATOMID = "atomId"
    DATEPROCESSED = "dateProcessed"
    CONNECTIONNAME = "connectionName"
    OPERATIONNAME = "operationName"
    ERRORMESSAGE = "errorMessage"
    STATUS = "status"
    DOCUMENTINDEX = "documentIndex"
    INCREMENTALDOCUMENTINDEX = "incrementalDocumentIndex"
    SIZE = "size"
    STARTSHAPE = "startShape"
    RETRYABLE = "retryable"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                GenericConnectorRecordSimpleExpressionProperty._member_map_.values(),
            )
        )


@JsonMap({})
class GenericConnectorRecordSimpleExpression(BaseModel):
    """GenericConnectorRecordSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: GenericConnectorRecordSimpleExpressionOperator
    :param property: property
    :type property: GenericConnectorRecordSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: GenericConnectorRecordSimpleExpressionOperator,
        property: GenericConnectorRecordSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """GenericConnectorRecordSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: GenericConnectorRecordSimpleExpressionOperator
        :param property: property
        :type property: GenericConnectorRecordSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, GenericConnectorRecordSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, GenericConnectorRecordSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .account_user_role_simple_expression import AccountUserRoleSimpleExpression
from .account_user_role_grouping_expression import AccountUserRoleGroupingExpression


class AccountUserRoleExpressionGuard(OneOfBaseModel):
    class_list = {
        "AccountUserRoleSimpleExpression": AccountUserRoleSimpleExpression,
        "AccountUserRoleGroupingExpression": AccountUserRoleGroupingExpression,
    }


AccountUserRoleExpression = Union[
    AccountUserRoleSimpleExpression, AccountUserRoleGroupingExpression
]




from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .observability_endpoint import ObservabilityEndpoint


class LogFilterType(Enum):
    """An enumeration representing different categories.

    :cvar RUNTIME: "RUNTIME"
    :vartype RUNTIME: str
    :cvar RUNNER: "RUNNER"
    :vartype RUNNER: str
    :cvar WORKER: "WORKER"
    :vartype WORKER: str
    :cvar BROWSER: "BROWSER"
    :vartype BROWSER: str
    :cvar PROCESS: "PROCESS"
    :vartype PROCESS: str
    :cvar DOCUMENT: "DOCUMENT"
    :vartype DOCUMENT: str
    """

    RUNTIME = "RUNTIME"
    RUNNER = "RUNNER"
    WORKER = "WORKER"
    BROWSER = "BROWSER"
    PROCESS = "PROCESS"
    DOCUMENT = "DOCUMENT"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, LogFilterType._member_map_.values()))


@JsonMap(
    {
        "log_batch_size": "logBatchSize",
        "log_flush_interval": "logFlushInterval",
        "log_max_queue_size": "logMaxQueueSize",
        "log_timeout_seconds": "logTimeoutSeconds",
        "observability_endpoint": "observabilityEndpoint",
        "log_filter_enabled": "logFilterEnabled",
        "log_filter_types": "logFilterTypes",
        "override_general_observability_endpoint": "overrideGeneralObservabilityEndpoint",
    }
)
class LogSettings(BaseModel):
    """LogSettings

    :param log_batch_size: log_batch_size
    :type log_batch_size: int
    :param log_flush_interval: log_flush_interval
    :type log_flush_interval: int
    :param log_max_queue_size: log_max_queue_size
    :type log_max_queue_size: int
    :param log_timeout_seconds: log_timeout_seconds
    :type log_timeout_seconds: int
    :param observability_endpoint: observability_endpoint
    :type observability_endpoint: ObservabilityEndpoint
    :param enabled: enabled, defaults to None
    :type enabled: bool, optional
    :param log_filter_enabled: log_filter_enabled, defaults to None
    :type log_filter_enabled: bool, optional
    :param log_filter_types: log_filter_types, defaults to None
    :type log_filter_types: List[str], optional
    :param override_general_observability_endpoint: override_general_observability_endpoint, defaults to None
    :type override_general_observability_endpoint: bool, optional
    """

    def __init__(
        self,
        log_batch_size: int,
        log_flush_interval: int,
        log_max_queue_size: int,
        log_timeout_seconds: int,
        observability_endpoint: ObservabilityEndpoint,
        enabled: bool = SENTINEL,
        log_filter_enabled: bool = SENTINEL,
        log_filter_types: List[str] = SENTINEL,
        override_general_observability_endpoint: bool = SENTINEL,
        **kwargs,
    ):
        """LogSettings

        :param log_batch_size: log_batch_size
        :type log_batch_size: int
        :param log_flush_interval: log_flush_interval
        :type log_flush_interval: int
        :param log_max_queue_size: log_max_queue_size
        :type log_max_queue_size: int
        :param log_timeout_seconds: log_timeout_seconds
        :type log_timeout_seconds: int
        :param observability_endpoint: observability_endpoint
        :type observability_endpoint: ObservabilityEndpoint
        :param enabled: enabled, defaults to None
        :type enabled: bool, optional
        :param log_filter_enabled: log_filter_enabled, defaults to None
        :type log_filter_enabled: bool, optional
        :param log_filter_types: log_filter_types, defaults to None
        :type log_filter_types: List[str], optional
        :param override_general_observability_endpoint: override_general_observability_endpoint, defaults to None
        :type override_general_observability_endpoint: bool, optional
        """
        self.log_batch_size = log_batch_size
        self.log_flush_interval = log_flush_interval
        self.log_max_queue_size = log_max_queue_size
        self.log_timeout_seconds = log_timeout_seconds
        self.observability_endpoint = self._define_object(
            observability_endpoint, ObservabilityEndpoint
        )
        if enabled is not SENTINEL:
            self.enabled = enabled
        if log_filter_enabled is not SENTINEL:
            self.log_filter_enabled = log_filter_enabled
        if log_filter_types is not SENTINEL:
            self.log_filter_types = log_filter_types
        if override_general_observability_endpoint is not SENTINEL:
            self.override_general_observability_endpoint = (
                override_general_observability_endpoint
            )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import ConnectionLicensingDownload, ConnectionLicensingReport


class ConnectionLicensingReportService(BaseService):

    @cast_models
    def create_connection_licensing_report(
        self, request_body: ConnectionLicensingReport = None
    ) -> Union[ConnectionLicensingDownload, str]:
        """Returns the Connection Licensing URL in the response to view or download the deployed connection details. To download connection licensing data for a given connector class:

         a. Send a POST and request body to `https://api.boomi.com/api/rest/v1/<accountId>/ConnectionLicensingReport`

         where accountId is the ID of the authenticating account for the request.

         Populate the request body with or without the available filters for the report that you want to download.

         b. Next, send a GET request using the URL returned in the POST response. The GET does not require a request body, and returns the deployed connection details.

         >**Note:** Do not pass any filters in the CREATE payload. This will not help get the Test & Production connections deployed details for all the connector classes. To get the Test and Production deployed connection details you have to pass ONLY the structure in the CREATE request, without any filters.

         - To apply multiple filters, add the Operator to the payload. Refer to the provided code samples for guidance.

         - The argument values for the *property* filters in the CREATE payload should be:

          - componentId - Must be a valid componentId value. If an invalid value is passed, the report or the GET response will be blank or will have zero records.

          - environmentId - Must be valid environmentId value. If an invalid value is passed, the report or the GET response will be blank or will have zero records.

          - containerId - Must be a valid atomId or moleculeId. If an invalid value is passed, the report or the GET response will be blank or will have zero records.

          - connectorClass - Must be valid connectorClass. Values must be either Standard, Small Business, Trading Partner, or Enterprise.

        :param request_body: The request body., defaults to None
        :type request_body: ConnectionLicensingReport, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ConnectionLicensingDownload, str]
        """

        Validator(ConnectionLicensingReport).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ConnectionLicensingReport",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ConnectionLicensingDownload._unmap(response)
        if content == "application/xml":
            return ConnectionLicensingDownload._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .simple_lookup_table import SimpleLookupTable


@JsonMap({"table": "Table"})
class MapExtensionsSimpleLookup(BaseModel):
    """MapExtensionsSimpleLookup

    :param table: table
    :type table: SimpleLookupTable
    """

    def __init__(self, table: SimpleLookupTable, **kwargs):
        """MapExtensionsSimpleLookup

        :param table: table
        :type table: SimpleLookupTable
        """
        self.table = self._define_object(table, SimpleLookupTable)
        self._kwargs = kwargs
